"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from enum import Enum
from typing import Any, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.pretrained_embedding_model_name import PretrainedEmbeddingModelName
from ..types import UNSET, Unset

T = TypeVar("T", bound="CloneLabeledMemorysetRequest")


@_attrs_define
class CloneLabeledMemorysetRequest:
    """
    Attributes:
        name (str):
        description (Union[None, Unset, str]):
        pretrained_embedding_model_name (Union[None, PretrainedEmbeddingModelName, Unset]):
        finetuned_embedding_model_id (Union[None, Unset, str]):
        max_seq_length_override (Union[None, Unset, int]):
    """

    name: str
    description: Union[None, Unset, str] = UNSET
    pretrained_embedding_model_name: Union[None, PretrainedEmbeddingModelName, Unset] = UNSET
    finetuned_embedding_model_id: Union[None, Unset, str] = UNSET
    max_seq_length_override: Union[None, Unset, int] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        pretrained_embedding_model_name: Union[None, Unset, str]
        if isinstance(self.pretrained_embedding_model_name, Unset):
            pretrained_embedding_model_name = UNSET
        elif isinstance(self.pretrained_embedding_model_name, PretrainedEmbeddingModelName):
            pretrained_embedding_model_name = (
                self.pretrained_embedding_model_name.value
                if isinstance(self.pretrained_embedding_model_name, Enum)
                else self.pretrained_embedding_model_name
            )
        else:
            pretrained_embedding_model_name = self.pretrained_embedding_model_name

        finetuned_embedding_model_id: Union[None, Unset, str]
        if isinstance(self.finetuned_embedding_model_id, Unset):
            finetuned_embedding_model_id = UNSET
        else:
            finetuned_embedding_model_id = self.finetuned_embedding_model_id

        max_seq_length_override: Union[None, Unset, int]
        if isinstance(self.max_seq_length_override, Unset):
            max_seq_length_override = UNSET
        else:
            max_seq_length_override = self.max_seq_length_override

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if pretrained_embedding_model_name is not UNSET:
            field_dict["pretrained_embedding_model_name"] = pretrained_embedding_model_name
        if finetuned_embedding_model_id is not UNSET:
            field_dict["finetuned_embedding_model_id"] = finetuned_embedding_model_id
        if max_seq_length_override is not UNSET:
            field_dict["max_seq_length_override"] = max_seq_length_override

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_pretrained_embedding_model_name(data: object) -> Union[None, PretrainedEmbeddingModelName, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                pretrained_embedding_model_name_type_0 = PretrainedEmbeddingModelName(data)

                return pretrained_embedding_model_name_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, PretrainedEmbeddingModelName, Unset], data)

        pretrained_embedding_model_name = _parse_pretrained_embedding_model_name(
            d.pop("pretrained_embedding_model_name", UNSET)
        )

        def _parse_finetuned_embedding_model_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        finetuned_embedding_model_id = _parse_finetuned_embedding_model_id(d.pop("finetuned_embedding_model_id", UNSET))

        def _parse_max_seq_length_override(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        max_seq_length_override = _parse_max_seq_length_override(d.pop("max_seq_length_override", UNSET))

        clone_labeled_memoryset_request = cls(
            name=name,
            description=description,
            pretrained_embedding_model_name=pretrained_embedding_model_name,
            finetuned_embedding_model_id=finetuned_embedding_model_id,
            max_seq_length_override=max_seq_length_override,
        )

        clone_labeled_memoryset_request.additional_properties = d
        return clone_labeled_memoryset_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.labeled_memory_metrics import LabeledMemoryMetrics
    from ..models.labeled_memory_update_metadata_type_0 import LabeledMemoryUpdateMetadataType0


T = TypeVar("T", bound="LabeledMemoryUpdate")


@_attrs_define
class LabeledMemoryUpdate:
    """Model for updating a labeled memory (unset fields are kept as is)

    Attributes:
        memory_id (str):
        value (Union[Unset, str]):
        metadata (Union['LabeledMemoryUpdateMetadataType0', None, Unset]):
        source_id (Union[None, Unset, str]):
        label (Union[Unset, int]):
        metrics (Union['LabeledMemoryMetrics', None, Unset]):
    """

    memory_id: str
    value: Union[Unset, str] = UNSET
    metadata: Union["LabeledMemoryUpdateMetadataType0", None, Unset] = UNSET
    source_id: Union[None, Unset, str] = UNSET
    label: Union[Unset, int] = UNSET
    metrics: Union["LabeledMemoryMetrics", None, Unset] = UNSET

    def to_dict(self) -> dict[str, Any]:
        from ..models.labeled_memory_metrics import LabeledMemoryMetrics
        from ..models.labeled_memory_update_metadata_type_0 import LabeledMemoryUpdateMetadataType0

        memory_id = self.memory_id

        value: Union[Unset, str]
        if isinstance(self.value, Unset):
            value = UNSET
        else:
            value = self.value

        metadata: Union[Dict[str, Any], None, Unset]
        if isinstance(self.metadata, Unset):
            metadata = UNSET
        elif isinstance(self.metadata, LabeledMemoryUpdateMetadataType0):
            metadata = self.metadata.to_dict()
        else:
            metadata = self.metadata

        source_id: Union[None, Unset, str]
        if isinstance(self.source_id, Unset):
            source_id = UNSET
        else:
            source_id = self.source_id

        label = self.label

        metrics: Union[Dict[str, Any], None, Unset]
        if isinstance(self.metrics, Unset):
            metrics = UNSET
        elif isinstance(self.metrics, LabeledMemoryMetrics):
            metrics = self.metrics.to_dict()
        else:
            metrics = self.metrics

        field_dict: dict[str, Any] = {}
        field_dict.update(
            {
                "memory_id": memory_id,
            }
        )
        if value is not UNSET:
            field_dict["value"] = value
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if source_id is not UNSET:
            field_dict["source_id"] = source_id
        if label is not UNSET:
            field_dict["label"] = label
        if metrics is not UNSET:
            field_dict["metrics"] = metrics

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        from ..models.labeled_memory_metrics import LabeledMemoryMetrics
        from ..models.labeled_memory_update_metadata_type_0 import LabeledMemoryUpdateMetadataType0

        d = src_dict.copy()
        memory_id = d.pop("memory_id")

        def _parse_value(data: object) -> Union[Unset, str]:
            if isinstance(data, Unset):
                return data
            return cast(Union[Unset, str], data)

        value = _parse_value(d.pop("value", UNSET))

        def _parse_metadata(data: object) -> Union["LabeledMemoryUpdateMetadataType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metadata_type_0 = LabeledMemoryUpdateMetadataType0.from_dict(data)

                return metadata_type_0
            except:  # noqa: E722
                pass
            return cast(Union["LabeledMemoryUpdateMetadataType0", None, Unset], data)

        metadata = _parse_metadata(d.pop("metadata", UNSET))

        def _parse_source_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        source_id = _parse_source_id(d.pop("source_id", UNSET))

        label = d.pop("label", UNSET)

        def _parse_metrics(data: object) -> Union["LabeledMemoryMetrics", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                metrics_type_0 = LabeledMemoryMetrics.from_dict(data)

                return metrics_type_0
            except:  # noqa: E722
                pass
            return cast(Union["LabeledMemoryMetrics", None, Unset], data)

        metrics = _parse_metrics(d.pop("metrics", UNSET))

        labeled_memory_update = cls(
            memory_id=memory_id,
            value=value,
            metadata=metadata,
            source_id=source_id,
            label=label,
            metrics=metrics,
        )

        return labeled_memory_update



from enum import Enum


class FilterItemOp(str, Enum):
    IN = "in"
    LIKE = "like"
    NOT_IN = "not in"
    VALUE_0 = "=="
    VALUE_1 = "!="
    VALUE_2 = ">"
    VALUE_3 = ">="
    VALUE_4 = "<"
    VALUE_5 = "<="

    def __str__(self) -> str:
        return str(self.value)



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import TYPE_CHECKING, Any, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.labeled_memory_with_feedback_metrics import LabeledMemoryWithFeedbackMetrics


T = TypeVar("T", bound="PaginatedLabeledMemoryWithFeedbackMetrics")


@_attrs_define
class PaginatedLabeledMemoryWithFeedbackMetrics:
    """
    Attributes:
        items (List['LabeledMemoryWithFeedbackMetrics']):
        total (int):
        offset (int):
        limit (int):
    """

    items: List["LabeledMemoryWithFeedbackMetrics"]
    total: int
    offset: int
    limit: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        items = []
        for items_item_data in self.items:
            items_item = items_item_data.to_dict()
            items.append(items_item)

        total = self.total

        offset = self.offset

        limit = self.limit

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "items": items,
                "total": total,
                "offset": offset,
                "limit": limit,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        from ..models.labeled_memory_with_feedback_metrics import LabeledMemoryWithFeedbackMetrics

        d = src_dict.copy()
        items = []
        _items = d.pop("items")
        for items_item_data in _items:
            items_item = LabeledMemoryWithFeedbackMetrics.from_dict(items_item_data)

            items.append(items_item)

        total = d.pop("total")

        offset = d.pop("offset")

        limit = d.pop("limit")

        paginated_labeled_memory_with_feedback_metrics = cls(
            items=items,
            total=total,
            offset=offset,
            limit=limit,
        )

        paginated_labeled_memory_with_feedback_metrics.additional_properties = d
        return paginated_labeled_memory_with_feedback_metrics

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import Any, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.rac_head_type import RACHeadType
from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateRACModelRequest")


@_attrs_define
class CreateRACModelRequest:
    """
    Attributes:
        name (str):
        description (Union[None, Unset, str]):
        memoryset_id (Union[None, Unset, str]):
        memoryset_name (Union[None, Unset, str]):
        head_type (Union[Unset, RACHeadType]):
        weigh_memories (Union[None, Unset, bool]):
        min_memory_weight (Union[None, Unset, float]):
        memory_lookup_count (Union[None, Unset, int]):
        num_classes (Union[None, Unset, int]):
    """

    name: str
    description: Union[None, Unset, str] = UNSET
    memoryset_id: Union[None, Unset, str] = UNSET
    memoryset_name: Union[None, Unset, str] = UNSET
    head_type: Union[Unset, RACHeadType] = UNSET
    weigh_memories: Union[None, Unset, bool] = UNSET
    min_memory_weight: Union[None, Unset, float] = UNSET
    memory_lookup_count: Union[None, Unset, int] = UNSET
    num_classes: Union[None, Unset, int] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        memoryset_id: Union[None, Unset, str]
        if isinstance(self.memoryset_id, Unset):
            memoryset_id = UNSET
        else:
            memoryset_id = self.memoryset_id

        memoryset_name: Union[None, Unset, str]
        if isinstance(self.memoryset_name, Unset):
            memoryset_name = UNSET
        else:
            memoryset_name = self.memoryset_name

        head_type: Union[Unset, str] = UNSET
        if not isinstance(self.head_type, Unset):
            head_type = self.head_type.value

        weigh_memories: Union[None, Unset, bool]
        if isinstance(self.weigh_memories, Unset):
            weigh_memories = UNSET
        else:
            weigh_memories = self.weigh_memories

        min_memory_weight: Union[None, Unset, float]
        if isinstance(self.min_memory_weight, Unset):
            min_memory_weight = UNSET
        else:
            min_memory_weight = self.min_memory_weight

        memory_lookup_count: Union[None, Unset, int]
        if isinstance(self.memory_lookup_count, Unset):
            memory_lookup_count = UNSET
        else:
            memory_lookup_count = self.memory_lookup_count

        num_classes: Union[None, Unset, int]
        if isinstance(self.num_classes, Unset):
            num_classes = UNSET
        else:
            num_classes = self.num_classes

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if memoryset_id is not UNSET:
            field_dict["memoryset_id"] = memoryset_id
        if memoryset_name is not UNSET:
            field_dict["memoryset_name"] = memoryset_name
        if head_type is not UNSET:
            field_dict["head_type"] = head_type
        if weigh_memories is not UNSET:
            field_dict["weigh_memories"] = weigh_memories
        if min_memory_weight is not UNSET:
            field_dict["min_memory_weight"] = min_memory_weight
        if memory_lookup_count is not UNSET:
            field_dict["memory_lookup_count"] = memory_lookup_count
        if num_classes is not UNSET:
            field_dict["num_classes"] = num_classes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_memoryset_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        memoryset_id = _parse_memoryset_id(d.pop("memoryset_id", UNSET))

        def _parse_memoryset_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        memoryset_name = _parse_memoryset_name(d.pop("memoryset_name", UNSET))

        _head_type = d.pop("head_type", UNSET)
        head_type: Union[Unset, RACHeadType]
        if isinstance(_head_type, Unset):
            head_type = UNSET
        else:
            head_type = RACHeadType(_head_type)

        def _parse_weigh_memories(data: object) -> Union[None, Unset, bool]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool], data)

        weigh_memories = _parse_weigh_memories(d.pop("weigh_memories", UNSET))

        def _parse_min_memory_weight(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        min_memory_weight = _parse_min_memory_weight(d.pop("min_memory_weight", UNSET))

        def _parse_memory_lookup_count(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        memory_lookup_count = _parse_memory_lookup_count(d.pop("memory_lookup_count", UNSET))

        def _parse_num_classes(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        num_classes = _parse_num_classes(d.pop("num_classes", UNSET))

        create_rac_model_request = cls(
            name=name,
            description=description,
            memoryset_id=memoryset_id,
            memoryset_name=memoryset_name,
            head_type=head_type,
            weigh_memories=weigh_memories,
            min_memory_weight=min_memory_weight,
            memory_lookup_count=memory_lookup_count,
            num_classes=num_classes,
        )

        create_rac_model_request.additional_properties = d
        return create_rac_model_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from uuid import uuid4

import pytest

from .datasource import Datasource
from .embedding_model import (
    FinetunedEmbeddingModel,
    PretrainedEmbeddingModel,
    PretrainedEmbeddingModelName,
    TaskStatus,
)
from .memoryset import LabeledMemoryset


def test_open_pretrained_model():
    model = PretrainedEmbeddingModel.GTE_BASE
    assert model is not None
    assert isinstance(model, PretrainedEmbeddingModel)
    assert model.name == "GTE_BASE"
    assert model.embedding_dim == 768
    assert model.max_seq_length == 8192
    assert model is PretrainedEmbeddingModel.GTE_BASE


def test_open_pretrained_model_unauthenticated(unauthenticated):
    with pytest.raises(ValueError, match="Invalid API key"):
        PretrainedEmbeddingModel.GTE_BASE.embed("I love this airline")


def test_open_pretrained_model_not_found():
    with pytest.raises(LookupError):
        PretrainedEmbeddingModel._get("INVALID_MODEL")


def test_all_pretrained_models():
    models = PretrainedEmbeddingModel.all()
    assert len(models) == len(PretrainedEmbeddingModelName)
    assert all(m.name in PretrainedEmbeddingModelName.__members__ for m in models)


def test_embed_text():
    embedding = PretrainedEmbeddingModel.GTE_BASE.embed("I love this airline", max_seq_length=32)
    assert embedding is not None
    assert isinstance(embedding, list)
    assert len(embedding) == 768
    assert isinstance(embedding[0], float)


def test_embed_text_unauthenticated(unauthenticated):
    with pytest.raises(ValueError, match="Invalid API key"):
        PretrainedEmbeddingModel.GTE_BASE.embed("I love this airline", max_seq_length=32)


@pytest.fixture(scope="session")
def finetuned_model(datasource) -> FinetunedEmbeddingModel:
    return PretrainedEmbeddingModel.DISTILBERT.finetune("test_finetuned_model", datasource, value_column="text")


def test_finetune_model_with_datasource(finetuned_model: FinetunedEmbeddingModel):
    assert finetuned_model is not None
    assert finetuned_model.name == "test_finetuned_model"
    assert finetuned_model.base_model == PretrainedEmbeddingModel.DISTILBERT
    assert finetuned_model.embedding_dim == 768
    assert finetuned_model.max_seq_length == 512
    assert finetuned_model._status == TaskStatus.COMPLETED


def test_finetune_model_with_memoryset(memoryset: LabeledMemoryset):
    finetuned_model = PretrainedEmbeddingModel.DISTILBERT.finetune("test_finetuned_model_from_memoryset", memoryset)
    assert finetuned_model is not None
    assert finetuned_model.name == "test_finetuned_model_from_memoryset"
    assert finetuned_model.base_model == PretrainedEmbeddingModel.DISTILBERT
    assert finetuned_model.embedding_dim == 768
    assert finetuned_model.max_seq_length == 512
    assert finetuned_model._status == TaskStatus.COMPLETED


def test_finetune_model_already_exists_error(datasource: Datasource, finetuned_model):
    with pytest.raises(ValueError):
        PretrainedEmbeddingModel.DISTILBERT.finetune("test_finetuned_model", datasource, value_column="text")


def test_finetune_model_already_exists_return(datasource: Datasource, finetuned_model):
    with pytest.raises(ValueError):
        PretrainedEmbeddingModel.GTE_BASE.finetune(
            "test_finetuned_model", datasource, if_exists="open", value_column="text"
        )

    new_model = PretrainedEmbeddingModel.DISTILBERT.finetune(
        "test_finetuned_model", datasource, if_exists="open", value_column="text"
    )
    assert new_model is not None
    assert new_model.name == "test_finetuned_model"
    assert new_model.base_model == PretrainedEmbeddingModel.DISTILBERT
    assert new_model.embedding_dim == 768
    assert new_model.max_seq_length == 512
    assert new_model._status == TaskStatus.COMPLETED


def test_finetune_model_unauthenticated(unauthenticated, datasource: Datasource):
    with pytest.raises(ValueError, match="Invalid API key"):
        PretrainedEmbeddingModel.DISTILBERT.finetune(
            "test_finetuned_model_unauthenticated", datasource, value_column="text"
        )


def test_use_finetuned_model_in_memoryset(datasource: Datasource, finetuned_model: FinetunedEmbeddingModel):
    memoryset = LabeledMemoryset.create(
        "test_memoryset_finetuned_model",
        datasource,
        embedding_model=finetuned_model,
        value_column="text",
    )
    assert memoryset is not None
    assert memoryset.name == "test_memoryset_finetuned_model"
    assert memoryset.embedding_model == finetuned_model
    assert memoryset.length == datasource.length


def test_open_finetuned_model(finetuned_model: FinetunedEmbeddingModel):
    model = FinetunedEmbeddingModel.open(finetuned_model.name)
    assert isinstance(model, FinetunedEmbeddingModel)
    assert model.id == finetuned_model.id
    assert model.name == finetuned_model.name
    assert model.base_model == PretrainedEmbeddingModel.DISTILBERT
    assert model.embedding_dim == 768
    assert model.max_seq_length == 512
    assert model == finetuned_model


def test_embed_finetuned_model(finetuned_model: FinetunedEmbeddingModel):
    embedding = finetuned_model.embed("I love this airline")
    assert embedding is not None
    assert isinstance(embedding, list)
    assert len(embedding) == 768
    assert isinstance(embedding[0], float)


def test_all_finetuned_models(finetuned_model: FinetunedEmbeddingModel):
    models = FinetunedEmbeddingModel.all()
    assert len(models) > 0
    assert any(model.name == finetuned_model.name for model in models)


def test_all_finetuned_models_unauthenticated(unauthenticated):
    with pytest.raises(ValueError, match="Invalid API key"):
        FinetunedEmbeddingModel.all()


def test_all_finetuned_models_unauthorized(unauthorized, finetuned_model: FinetunedEmbeddingModel):
    assert finetuned_model not in FinetunedEmbeddingModel.all()


def test_drop_finetuned_model(datasource: Datasource):
    PretrainedEmbeddingModel.DISTILBERT.finetune("finetuned_model_to_delete", datasource, value_column="text")
    assert FinetunedEmbeddingModel.open("finetuned_model_to_delete")
    FinetunedEmbeddingModel.drop("finetuned_model_to_delete")
    with pytest.raises(LookupError):
        FinetunedEmbeddingModel.open("finetuned_model_to_delete")


def test_drop_finetuned_model_unauthenticated(unauthenticated, datasource: Datasource):
    with pytest.raises(ValueError, match="Invalid API key"):
        PretrainedEmbeddingModel.DISTILBERT.finetune("finetuned_model_to_delete", datasource, value_column="text")


def test_drop_finetuned_model_not_found():
    with pytest.raises(LookupError):
        FinetunedEmbeddingModel.drop(str(uuid4()))
    # ignores error if specified
    FinetunedEmbeddingModel.drop(str(uuid4()), if_not_exists="ignore")


def test_drop_finetuned_model_unauthorized(unauthorized, finetuned_model: FinetunedEmbeddingModel):
    with pytest.raises(LookupError):
        FinetunedEmbeddingModel.drop(finetuned_model.id)



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from enum import Enum
from typing import Any, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.column_type import ColumnType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ColumnInfo")


@_attrs_define
class ColumnInfo:
    """Information about a column in a datasource

    Attributes:
        name (str):
        type (ColumnType): The type of a column in a datasource
        enum_options (Union[List[str], None, Unset]):
    """

    name: str
    type: ColumnType
    enum_options: Union[List[str], None, Unset] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        type = self.type.value if isinstance(self.type, Enum) else self.type

        enum_options: Union[List[str], None, Unset]
        if isinstance(self.enum_options, Unset):
            enum_options = UNSET
        elif isinstance(self.enum_options, list):
            enum_options = self.enum_options

        else:
            enum_options = self.enum_options

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "type": type,
            }
        )
        if enum_options is not UNSET:
            field_dict["enum_options"] = enum_options

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        type = ColumnType(d.pop("type"))

        def _parse_enum_options(data: object) -> Union[List[str], None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                enum_options_type_0 = cast(List[str], data)

                return enum_options_type_0
            except:  # noqa: E722
                pass
            return cast(Union[List[str], None, Unset], data)

        enum_options = _parse_enum_options(d.pop("enum_options", UNSET))

        column_info = cls(
            name=name,
            type=type,
            enum_options=enum_options,
        )

        column_info.additional_properties = d
        return column_info

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class NotFoundErrorResponseResourceType0(str, Enum):
    ANALYSIS = "analysis"
    API_KEY = "api_key"
    CLASSIFICATION_MODEL = "classification_model"
    DATASOURCE = "datasource"
    EMBEDDING_EVALUATION = "embedding_evaluation"
    EVALUATION = "evaluation"
    FEEDBACK_CATEGORY = "feedback_category"
    FINETUNED_EMBEDDING_MODEL = "finetuned_embedding_model"
    MEMORY = "memory"
    MEMORYSET = "memoryset"
    ORG = "org"
    PREDICTION = "prediction"
    PRETRAINED_EMBEDDING_MODEL = "pretrained_embedding_model"
    TASK = "task"

    def __str__(self) -> str:
        return str(self.value)



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import Any, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="LabelPredictionResult")


@_attrs_define
class LabelPredictionResult:
    """Predicted label and confidence for a single input.

    Attributes:
        prediction_id (str):
        confidence (float):
        anomaly_score (Union[None, float]):
        label (int):
        label_name (Union[None, str]):
    """

    prediction_id: str
    confidence: float
    anomaly_score: Union[None, float]
    label: int
    label_name: Union[None, str]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        prediction_id = self.prediction_id

        confidence = self.confidence

        anomaly_score: Union[None, float]
        anomaly_score = self.anomaly_score

        label = self.label

        label_name: Union[None, str]
        label_name = self.label_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "prediction_id": prediction_id,
                "confidence": confidence,
                "anomaly_score": anomaly_score,
                "label": label,
                "label_name": label_name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        prediction_id = d.pop("prediction_id")

        confidence = d.pop("confidence")

        def _parse_anomaly_score(data: object) -> Union[None, float]:
            if data is None:
                return data
            return cast(Union[None, float], data)

        anomaly_score = _parse_anomaly_score(d.pop("anomaly_score"))

        label = d.pop("label")

        def _parse_label_name(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        label_name = _parse_label_name(d.pop("label_name"))

        label_prediction_result = cls(
            prediction_id=prediction_id,
            confidence=confidence,
            anomaly_score=anomaly_score,
            label=label,
            label_name=label_name,
        )

        label_prediction_result.additional_properties = d
        return label_prediction_result

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.precision_recall_curve import PrecisionRecallCurve
    from ..models.roc_curve import ROCCurve


T = TypeVar("T", bound="ClassificationEvaluationResult")


@_attrs_define
class ClassificationEvaluationResult:
    """
    Attributes:
        f1_score (float):
        accuracy (float):
        loss (float):
        precision_recall_curve (Union['PrecisionRecallCurve', None]):
        roc_curve (Union['ROCCurve', None]):
    """

    f1_score: float
    accuracy: float
    loss: float
    precision_recall_curve: Union["PrecisionRecallCurve", None]
    roc_curve: Union["ROCCurve", None]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.precision_recall_curve import PrecisionRecallCurve
        from ..models.roc_curve import ROCCurve

        f1_score = self.f1_score

        accuracy = self.accuracy

        loss = self.loss

        precision_recall_curve: Union[Dict[str, Any], None]
        if isinstance(self.precision_recall_curve, PrecisionRecallCurve):
            precision_recall_curve = self.precision_recall_curve.to_dict()
        else:
            precision_recall_curve = self.precision_recall_curve

        roc_curve: Union[Dict[str, Any], None]
        if isinstance(self.roc_curve, ROCCurve):
            roc_curve = self.roc_curve.to_dict()
        else:
            roc_curve = self.roc_curve

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "f1_score": f1_score,
                "accuracy": accuracy,
                "loss": loss,
                "precision_recall_curve": precision_recall_curve,
                "roc_curve": roc_curve,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        from ..models.precision_recall_curve import PrecisionRecallCurve
        from ..models.roc_curve import ROCCurve

        d = src_dict.copy()
        f1_score = d.pop("f1_score")

        accuracy = d.pop("accuracy")

        loss = d.pop("loss")

        def _parse_precision_recall_curve(data: object) -> Union["PrecisionRecallCurve", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                precision_recall_curve_type_0 = PrecisionRecallCurve.from_dict(data)

                return precision_recall_curve_type_0
            except:  # noqa: E722
                pass
            return cast(Union["PrecisionRecallCurve", None], data)

        precision_recall_curve = _parse_precision_recall_curve(d.pop("precision_recall_curve"))

        def _parse_roc_curve(data: object) -> Union["ROCCurve", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                roc_curve_type_0 = ROCCurve.from_dict(data)

                return roc_curve_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ROCCurve", None], data)

        roc_curve = _parse_roc_curve(d.pop("roc_curve"))

        classification_evaluation_result = cls(
            f1_score=f1_score,
            accuracy=accuracy,
            loss=loss,
            precision_recall_curve=precision_recall_curve,
            roc_curve=roc_curve,
        )

        classification_evaluation_result.additional_properties = d
        return classification_evaluation_result

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import pytest

from .classification_model import ClassificationModel
from .memoryset import LabeledMemoryLookup
from .telemetry import FeedbackCategory, LabelPrediction


def test_get_prediction(model: ClassificationModel):
    predictions = model.predict(["Do you love soup?", "Are cats cute?"])
    prediction_with_telemetry = LabelPrediction.get(predictions[0].prediction_id)
    assert prediction_with_telemetry is not None
    assert prediction_with_telemetry.label == 0
    assert prediction_with_telemetry.input_value == "Do you love soup?"


def test_get_predictions(model: ClassificationModel):
    predictions = model.predict(["Do you love soup?", "Are cats cute?"])
    prediction_with_telemetry = LabelPrediction.get([predictions[0].prediction_id, predictions[1].prediction_id])
    assert len(prediction_with_telemetry) == 2
    assert prediction_with_telemetry[0].label == 0
    assert prediction_with_telemetry[0].input_value == "Do you love soup?"
    assert prediction_with_telemetry[1].label == 1


def test_get_predictions_with_expected_label_match(model: ClassificationModel):
    model.predict(["Do you love soup?", "Are cats cute?"], expected_labels=[0, 0], tags={"expected_label_match"})
    model.predict("no expectations", tags={"expected_label_match"})
    assert len(model.predictions(tag="expected_label_match")) == 3
    assert len(model.predictions(expected_label_match=True, tag="expected_label_match")) == 1
    assert len(model.predictions(expected_label_match=False, tag="expected_label_match")) == 1


def test_get_prediction_memory_lookups(model: ClassificationModel):
    prediction = model.predict("Do you love soup?")
    assert isinstance(prediction.memory_lookups, list)
    assert len(prediction.memory_lookups) > 0
    assert all(isinstance(lookup, LabeledMemoryLookup) for lookup in prediction.memory_lookups)


def test_record_feedback(model: ClassificationModel):
    prediction = model.predict("Do you love soup?")
    assert "correct" not in prediction.feedback
    prediction.record_feedback(category="correct", value=prediction.label == 0)
    assert prediction.feedback["correct"] is True


def test_record_feedback_with_invalid_value(model: ClassificationModel):
    with pytest.raises(ValueError, match=r"Invalid input.*"):
        model.predict("Do you love soup?").record_feedback(category="correct", value="not a bool")  # type: ignore


def test_record_feedback_with_inconsistent_value_for_category(model: ClassificationModel):
    model.predict("Do you love soup?").record_feedback(category="correct", value=True)
    with pytest.raises(ValueError, match=r"Invalid input.*"):
        model.predict("Do you love soup?").record_feedback(category="correct", value=-1.0)


def test_delete_feedback(model: ClassificationModel):
    prediction = model.predict("Do you love soup?")
    prediction.record_feedback(category="test_delete", value=True)
    assert "test_delete" in prediction.feedback
    prediction.delete_feedback("test_delete")
    assert "test_delete" not in prediction.feedback


def test_list_feedback_categories(model: ClassificationModel):
    prediction = model.predict("Do you love soup?")
    prediction.record_feedback(category="correct", value=True)
    prediction.record_feedback(category="confidence", value=0.8)
    categories = FeedbackCategory.all()
    assert len(categories) >= 2
    assert any(c.name == "correct" and c.value_type == bool for c in categories)
    assert any(c.name == "confidence" and c.value_type == float for c in categories)


def test_drop_feedback_category(model: ClassificationModel):
    prediction = model.predict("Do you love soup?")
    prediction.record_feedback(category="test_category", value=True)
    assert any(c.name == "test_category" for c in FeedbackCategory.all())
    FeedbackCategory.drop("test_category")
    assert not any(c.name == "test_category" for c in FeedbackCategory.all())
    prediction.refresh()
    assert "test_category" not in prediction.feedback


def test_update_prediction(model: ClassificationModel):
    prediction = model.predict("Do you love soup?")
    assert prediction.expected_label is None
    assert prediction.tags == set()
    # update expected label
    prediction.update(expected_label=1)
    assert prediction.expected_label == 1

    # update tags
    prediction.update(tags={"test_tag1", "test_tag2"})
    assert prediction.tags == {"test_tag1", "test_tag2"}

    # update both
    prediction.update(expected_label=0, tags={"new_tag"})
    assert prediction.expected_label == 0
    assert prediction.tags == {"new_tag"}

    # remove expected label
    prediction.update(expected_label=None)
    assert prediction.expected_label is None

    # remove tags
    prediction.update(tags=None)
    assert prediction.tags == set()



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

import datetime
from enum import Enum
from typing import Any, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.create_api_key_response_scope_item import CreateApiKeyResponseScopeItem

T = TypeVar("T", bound="CreateApiKeyResponse")


@_attrs_define
class CreateApiKeyResponse:
    """
    Attributes:
        id (str):
        org_id (str):
        name (str):
        created_by (Union[None, str]):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        scope (List[CreateApiKeyResponseScopeItem]):
        api_key (str):
    """

    id: str
    org_id: str
    name: str
    created_by: Union[None, str]
    created_at: datetime.datetime
    updated_at: datetime.datetime
    scope: List[CreateApiKeyResponseScopeItem]
    api_key: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        org_id = self.org_id

        name = self.name

        created_by: Union[None, str]
        created_by = self.created_by

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        scope = []
        for scope_item_data in self.scope:
            scope_item = scope_item_data.value if isinstance(scope_item_data, Enum) else scope_item_data
            scope.append(scope_item)

        api_key = self.api_key

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "org_id": org_id,
                "name": name,
                "created_by": created_by,
                "created_at": created_at,
                "updated_at": updated_at,
                "scope": scope,
                "api_key": api_key,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        org_id = d.pop("org_id")

        name = d.pop("name")

        def _parse_created_by(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        created_by = _parse_created_by(d.pop("created_by"))

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        scope = []
        _scope = d.pop("scope")
        for scope_item_data in _scope:
            scope_item = CreateApiKeyResponseScopeItem(scope_item_data)

            scope.append(scope_item)

        api_key = d.pop("api_key")

        create_api_key_response = cls(
            id=id,
            org_id=org_id,
            name=name,
            created_by=created_by,
            created_at=created_at,
            updated_at=updated_at,
            scope=scope,
            api_key=api_key,
        )

        create_api_key_response.additional_properties = d
        return create_api_key_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import pickle
from dataclasses import asdict, is_dataclass
from os import PathLike
from typing import Any, cast

from datasets import Dataset
from torch.utils.data import DataLoader as TorchDataLoader
from torch.utils.data import Dataset as TorchDataset


def parse_dict_like(item: Any, column_names: list[str] | None = None) -> dict:
    if isinstance(item, dict):
        return item

    if isinstance(item, tuple):
        if column_names is not None:
            assert len(item) == len(column_names)
            return {column_names[i]: item[i] for i in range(len(item))}
        elif hasattr(item, "_fields") and all(isinstance(field, str) for field in item._fields):  # type: ignore
            return {field: getattr(item, field) for field in item._fields}  # type: ignore
        else:
            raise ValueError("For datasets that return unnamed tuples, please provide column_names argument")

    if is_dataclass(item) and not isinstance(item, type):
        return asdict(item)

    raise ValueError(f"Cannot parse {type(item)}")


def parse_batch(batch: Any, column_names: list[str] | None = None) -> list[dict]:
    if isinstance(batch, list):
        return [parse_dict_like(item, column_names) for item in batch]

    batch = parse_dict_like(batch, column_names)
    keys = list(batch.keys())
    batch_size = len(batch[keys[0]])
    for key in keys:
        if not len(batch[key]) == batch_size:
            raise ValueError(f"Batch must consist of values of the same length, but {key} has length {len(batch[key])}")
    return [{key: batch[key][idx] for key in keys} for idx in range(batch_size)]


def hf_dataset_from_torch(torch_data: TorchDataLoader | TorchDataset, column_names: list[str] | None = None) -> Dataset:
    if isinstance(torch_data, TorchDataLoader):
        dataloader = torch_data
    else:
        dataloader = TorchDataLoader(torch_data, batch_size=1, collate_fn=lambda x: x)

    def generator():
        for batch in dataloader:
            yield from parse_batch(batch, column_names=column_names)

    return cast(Dataset, Dataset.from_generator(generator))


def hf_dataset_from_disk(file_path: str | PathLike) -> Dataset:
    """
    Load a dataset from disk into a HuggingFace Dataset object.

    Params:
        file_path: Path to the file on disk to create the memoryset from. The file type will
                be inferred from the file extension. The following file types are supported:

                - .pkl: [`Pickle`][pickle] files containing lists of dictionaries or dictionaries of columns
                - .json/.jsonl: [`JSON`][json] and [`JSON`] Lines files
                - .csv: [`CSV`][csv] files
                - .parquet: [`Parquet`](https://arrow.apache.org/docs/python/generated/pyarrow.parquet.ParquetFile.html#pyarrow.parquet.ParquetFile) files
                - dataset directory: Directory containing a saved HuggingFace [`Dataset`][datasets.Dataset]

    Returns:
        A HuggingFace Dataset object containing the loaded data.

    Raises:
        [`ValueError`][ValueError]: If the pickle file contains unsupported data types or if
            loading the dataset fails for any reason.
    """
    if str(file_path).endswith(".pkl"):
        data = pickle.load(open(file_path, "rb"))
        if isinstance(data, list):
            return Dataset.from_list(data)
        elif isinstance(data, dict):
            return Dataset.from_dict(data)
        else:
            raise ValueError(f"Unsupported pickle file: {file_path}")
    elif str(file_path).endswith(".json"):
        hf_dataset = Dataset.from_json(file_path)
    elif str(file_path).endswith(".jsonl"):
        hf_dataset = Dataset.from_json(file_path)
    elif str(file_path).endswith(".csv"):
        hf_dataset = Dataset.from_csv(file_path)
    elif str(file_path).endswith(".parquet"):
        hf_dataset = Dataset.from_parquet(file_path)
    else:
        try:
            hf_dataset = Dataset.load_from_disk(file_path)
        except Exception as e:
            raise ValueError(f"Failed to load dataset from disk: {e}")

    return cast(Dataset, hf_dataset)



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

import json
from io import BytesIO
from typing import Any, List, Tuple, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, File, FileJsonType, Unset

T = TypeVar("T", bound="BodyCreateDatasourceDatasourcePost")


@_attrs_define
class BodyCreateDatasourceDatasourcePost:
    """
    Attributes:
        name (str):
        files (Union[List[File], None, Unset]):
        description (Union[None, Unset, str]):
        content (Union[None, Unset, str]):
    """

    name: str
    files: Union[List[File], None, Unset] = UNSET
    description: Union[None, Unset, str] = UNSET
    content: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        files: Union[List[FileJsonType], None, Unset]
        if isinstance(self.files, Unset):
            files = UNSET
        elif isinstance(self.files, list):
            files = []
            for files_type_0_item_data in self.files:
                files_type_0_item = files_type_0_item_data.to_tuple()

                files.append(files_type_0_item)

        else:
            files = self.files

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        content: Union[None, Unset, str]
        if isinstance(self.content, Unset):
            content = UNSET
        else:
            content = self.content

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if files is not UNSET:
            field_dict["files"] = files
        if description is not UNSET:
            field_dict["description"] = description
        if content is not UNSET:
            field_dict["content"] = content

        return field_dict

    def to_multipart(self) -> dict[str, Any]:
        name = (None, str(self.name).encode(), "text/plain")

        files: Union[Tuple[None, bytes, str], Unset]

        if isinstance(self.files, Unset):
            files = UNSET
        elif isinstance(self.files, list):
            _temp_files = []
            for files_type_0_item_data in self.files:
                files_type_0_item = files_type_0_item_data.to_tuple()

                _temp_files.append(files_type_0_item)
            files = (None, json.dumps(_temp_files).encode(), "application/json")
        else:
            files = (None, str(self.files).encode(), "text/plain")

        description: Union[Tuple[None, bytes, str], Unset]

        if isinstance(self.description, Unset):
            description = UNSET
        elif isinstance(self.description, str):
            description = (None, str(self.description).encode(), "text/plain")
        else:
            description = (None, str(self.description).encode(), "text/plain")

        content: Union[Tuple[None, bytes, str], Unset]

        if isinstance(self.content, Unset):
            content = UNSET
        elif isinstance(self.content, str):
            content = (None, str(self.content).encode(), "text/plain")
        else:
            content = (None, str(self.content).encode(), "text/plain")

        field_dict: dict[str, Any] = {}
        for prop_name, prop in self.additional_properties.items():
            field_dict[prop_name] = (None, str(prop).encode(), "text/plain")

        field_dict.update(
            {
                "name": name,
            }
        )
        if files is not UNSET:
            field_dict["files"] = files
        if description is not UNSET:
            field_dict["description"] = description
        if content is not UNSET:
            field_dict["content"] = content

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        def _parse_files(data: object) -> Union[List[File], None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                files_type_0 = []
                _files_type_0 = data
                for files_type_0_item_data in _files_type_0:
                    files_type_0_item = File(payload=BytesIO(files_type_0_item_data))

                    files_type_0.append(files_type_0_item)

                return files_type_0
            except:  # noqa: E722
                pass
            return cast(Union[List[File], None, Unset], data)

        files = _parse_files(d.pop("files", UNSET))

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_content(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        content = _parse_content(d.pop("content", UNSET))

        body_create_datasource_datasource_post = cls(
            name=name,
            files=files,
            description=description,
            content=content,
        )

        body_create_datasource_datasource_post.additional_properties = d
        return body_create_datasource_datasource_post

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import Any, List, Type, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="PrecisionRecallCurve")


@_attrs_define
class PrecisionRecallCurve:
    """
    Attributes:
        thresholds (List[float]):
        precisions (List[float]):
        recalls (List[float]):
        auc (float):
    """

    thresholds: List[float]
    precisions: List[float]
    recalls: List[float]
    auc: float
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        thresholds = self.thresholds

        precisions = self.precisions

        recalls = self.recalls

        auc = self.auc

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "thresholds": thresholds,
                "precisions": precisions,
                "recalls": recalls,
                "auc": auc,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        thresholds = cast(List[float], d.pop("thresholds"))

        precisions = cast(List[float], d.pop("precisions"))

        recalls = cast(List[float], d.pop("recalls"))

        auc = d.pop("auc")

        precision_recall_curve = cls(
            thresholds=thresholds,
            precisions=precisions,
            recalls=recalls,
            auc=auc,
        )

        precision_recall_curve.additional_properties = d
        return precision_recall_curve

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import logging
import tempfile
from datetime import datetime
from os import PathLike
from pathlib import Path
from typing import cast

import pandas as pd
import pyarrow as pa
from datasets import Dataset
from torch.utils.data import DataLoader as TorchDataLoader
from torch.utils.data import Dataset as TorchDataset

from ._generated_api_client.api import (
    delete_datasource,
    get_datasource,
    list_datasources,
)
from ._generated_api_client.api.datasource.create_datasource_datasource_post import (
    _parse_response as parse_create_response,
)
from ._generated_api_client.client import get_client
from ._generated_api_client.models import ColumnType, DatasourceMetadata
from ._utils.common import CreateMode, DropMode
from ._utils.data_parsing import hf_dataset_from_disk, hf_dataset_from_torch


class Datasource:
    """
    A Handle to a datasource in the OrcaCloud

    A Datasource is a collection of data saved to the OrcaCloud that can be used to create a [`Memoryset`][orca_sdk.LabeledMemoryset].
    It can be created from a Hugging Face Dataset, a PyTorch DataLoader or Dataset, a list of dictionaries, a dictionary of columns, a pandas DataFrame, a pyarrow Table, or a local file.

    Attributes:
        id: Unique identifier for the datasource
        name: Unique name of the datasource
        description: Optional description of the datasource
        length: Number of rows in the datasource
        created_at: When the datasource was created
        columns: Dictionary of column names and types
    """

    id: str
    name: str
    description: str | None
    length: int
    created_at: datetime
    updated_at: datetime
    columns: dict[str, str]

    def __init__(self, metadata: DatasourceMetadata):
        # for internal use only, do not document
        self.id = metadata.id
        self.name = metadata.name
        self.length = metadata.length
        self.created_at = metadata.created_at
        self.updated_at = metadata.updated_at
        self.description = metadata.description
        self.columns = {
            column.name: (
                f"enum({', '.join(f'{option!r}' for option in column.enum_options) if column.enum_options else ''})"
                if column.type == ColumnType.ENUM
                else "str" if column.type == ColumnType.STRING else column.type.value.lower()
            )
            for column in metadata.columns
        }

    def __eq__(self, other) -> bool:
        return isinstance(other, Datasource) and self.id == other.id

    def __repr__(self) -> str:
        return (
            "Datasource({\n"
            + f"    name: '{self.name}',\n"
            + f"    length: {self.length},\n"
            + "    columns: {{\n        "
            + "\n        ".join([f"{k}: {v}" for k, v in self.columns.items()])
            + "\n    }}\n"
            + "})"
        )

    @classmethod
    def from_hf_dataset(
        cls, name: str, dataset: Dataset, if_exists: CreateMode = "error", description: str | None = None
    ) -> Datasource:
        """
        Create a new datasource from a Hugging Face Dataset

        Params:
            name: Required name for the new datasource (must be unique)
            dataset: The Hugging Face Dataset to create the datasource from
            if_exists: What to do if a datasource with the same name already exists, defaults to
                `"error"`. Other option is `"open"` to open the existing datasource.
            description: Optional description for the datasource

        Returns:
            A handle to the new datasource in the OrcaCloud

        Raises:
            ValueError: If the datasource already exists and if_exists is `"error"`
        """
        client = get_client()

        if cls.exists(name):
            if if_exists == "error":
                raise ValueError(f"Dataset with name {name} already exists")
            elif if_exists == "open":
                return cls.open(name)

        with tempfile.TemporaryDirectory() as tmp_dir:
            dataset.save_to_disk(tmp_dir)
            files = []
            for file_path in Path(tmp_dir).iterdir():
                buffered_reader = open(file_path, "rb")
                files.append(("files", buffered_reader))

            # Do not use Generated client for this endpoint b/c it does not handle files properly
            metadata = parse_create_response(
                response=client.get_httpx_client().request(
                    method="post",
                    url="/datasource/",
                    files=files,
                    data={"name": name, "description": description},
                )
            )
            return cls(metadata=metadata)

    @classmethod
    def from_pytorch(
        cls,
        name: str,
        torch_data: TorchDataLoader | TorchDataset,
        column_names: list[str] | None = None,
        if_exists: CreateMode = "error",
        description: str | None = None,
    ) -> Datasource:
        """
        Create a new datasource from a PyTorch DataLoader or Dataset

        Params:
            name: Required name for the new datasource (must be unique)
            torch_data: The PyTorch DataLoader or Dataset to create the datasource from
            column_names: If the provided dataset or data loader returns unnamed tuples, this
                argument must be provided to specify the names of the columns.
            if_exists: What to do if a datasource with the same name already exists, defaults to
                `"error"`. Other option is `"open"` to open the existing datasource.
            description: Optional description for the datasource

        Returns:
            A handle to the new datasource in the OrcaCloud

        Raises:
            ValueError: If the datasource already exists and if_exists is `"error"`
        """
        hf_dataset = hf_dataset_from_torch(torch_data, column_names=column_names)
        return cls.from_hf_dataset(name, hf_dataset, if_exists=if_exists, description=description)

    @classmethod
    def from_list(
        cls, name: str, data: list[dict], if_exists: CreateMode = "error", description: str | None = None
    ) -> Datasource:
        """
        Create a new datasource from a list of dictionaries

        Params:
            name: Required name for the new datasource (must be unique)
            data: The list of dictionaries to create the datasource from
            if_exists: What to do if a datasource with the same name already exists, defaults to
                `"error"`. Other option is `"open"` to open the existing datasource.
            description: Optional description for the datasource

        Returns:
            A handle to the new datasource in the OrcaCloud

        Raises:
            ValueError: If the datasource already exists and if_exists is `"error"`

        Examples:
            >>> Datasource.from_list("my_datasource", [{"text": "Hello, world!", "label": 1}, {"text": "Goodbye", "label": 0}])
        """
        hf_dataset = Dataset.from_list(data)
        return cls.from_hf_dataset(name, hf_dataset, if_exists=if_exists, description=description)

    @classmethod
    def from_dict(
        cls, name: str, data: dict, if_exists: CreateMode = "error", description: str | None = None
    ) -> Datasource:
        """
        Create a new datasource from a dictionary of columns

        Params:
            name: Required name for the new datasource (must be unique)
            data: The dictionary of columns to create the datasource from
            if_exists: What to do if a datasource with the same name already exists, defaults to
                `"error"`. Other option is `"open"` to open the existing datasource.
            description: Optional description for the datasource

        Returns:
            A handle to the new datasource in the OrcaCloud

        Raises:
            ValueError: If the datasource already exists and if_exists is `"error"`

        Examples:
            >>> Datasource.from_dict("my_datasource", {"text": ["Hello, world!", "Goodbye"], "label": [1, 0]})
        """
        hf_dataset = Dataset.from_dict(data)
        return cls.from_hf_dataset(name, hf_dataset, if_exists=if_exists, description=description)

    @classmethod
    def from_pandas(
        cls, name: str, dataframe: pd.DataFrame, if_exists: CreateMode = "error", description: str | None = None
    ) -> Datasource:
        """
        Create a new datasource from a pandas DataFrame

        Params:
            name: Required name for the new datasource (must be unique)
            dataframe: The pandas DataFrame to create the datasource from
            if_exists: What to do if a datasource with the same name already exists, defaults to
                `"error"`. Other option is `"open"` to open the existing datasource.
            description: Optional description for the datasource

        Returns:
            A handle to the new datasource in the OrcaCloud

        Raises:
            ValueError: If the datasource already exists and if_exists is `"error"`
        """
        hf_dataset = Dataset.from_pandas(dataframe)
        return cls.from_hf_dataset(name, hf_dataset, if_exists=if_exists, description=description)

    @classmethod
    def from_arrow(
        cls, name: str, pyarrow_table: pa.Table, if_exists: CreateMode = "error", description: str | None = None
    ) -> Datasource:
        """
        Create a new datasource from a pyarrow Table

        Params:
            name: Required name for the new datasource (must be unique)
            pyarrow_table: The pyarrow Table to create the datasource from
            if_exists: What to do if a datasource with the same name already exists, defaults to
                `"error"`. Other option is `"open"` to open the existing datasource.
            description: Optional description for the datasource

        Returns:
            A handle to the new datasource in the OrcaCloud

        Raises:
            ValueError: If the datasource already exists and if_exists is `"error"`
        """
        hf_dataset = Dataset(pyarrow_table)
        return cls.from_hf_dataset(name, hf_dataset, if_exists=if_exists, description=description)

    @classmethod
    def from_disk(
        cls, name: str, file_path: str | PathLike, if_exists: CreateMode = "error", description: str | None = None
    ) -> Datasource:
        """
        Create a new datasource from a local file

        Params:
            name: Required name for the new datasource (must be unique)
            file_path: Path to the file on disk to create the datasource from. The file type will
                be inferred from the file extension. The following file types are supported:

                - .pkl: [`Pickle`][pickle] files containing lists of dictionaries or dictionaries of columns
                - .json/.jsonl: [`JSON`][json] and [`JSON`] Lines files
                - .csv: [`CSV`][csv] files
                - .parquet: [`Parquet`][pyarrow.parquet.ParquetFile] files
                - dataset directory: Directory containing a saved HuggingFace [`Dataset`][datasets.Dataset]

            if_exists: What to do if a datasource with the same name already exists, defaults to
                `"error"`. Other option is `"open"` to open the existing datasource.
            description: Optional description for the datasource

        Returns:
            A handle to the new datasource in the OrcaCloud

        Raises:
            ValueError: If the datasource already exists and if_exists is `"error"`
        """
        hf_dataset = hf_dataset_from_disk(file_path)
        return cls.from_hf_dataset(name, cast(Dataset, hf_dataset), if_exists=if_exists, description=description)

    @classmethod
    def open(cls, name: str) -> Datasource:
        """
        Get a handle to a datasource by name or id in the OrcaCloud

        Params:
            name: The name or unique identifier of the datasource to get

        Returns:
            A handle to the existing datasource in the OrcaCloud

        Raises:
            LookupError: If the datasource does not exist
        """
        return cls(get_datasource(name))

    @classmethod
    def exists(cls, name_or_id: str) -> bool:
        """
        Check if a datasource exists in the OrcaCloud

        Params:
            name_or_id: The name or id of the datasource to check

        Returns:
            `True` if the datasource exists, `False` otherwise
        """
        try:
            cls.open(name_or_id)
            return True
        except LookupError:
            return False

    @classmethod
    def all(cls) -> list[Datasource]:
        """
        List all datasource handles in the OrcaCloud

        Returns:
            A list of all datasource handles in the OrcaCloud
        """
        return [cls(metadata) for metadata in list_datasources()]

    @classmethod
    def drop(cls, name_or_id: str, if_not_exists: DropMode = "error") -> None:
        """
        Delete a datasource from the OrcaCloud

        Params:
            name_or_id: The name or id of the datasource to delete
            if_not_exists: What to do if the datasource does not exist, defaults to
                `"error"`. Other options are `"ignore"` to do nothing.

        Raises:
            LookupError: If the datasource does not exist and if_not_exists is `"error"`
        """
        try:
            delete_datasource(name_or_id)
            logging.info(f"Deleted datasource {name_or_id}")
        except LookupError:
            if if_not_exists == "error":
                raise

    def __len__(self) -> int:
        return self.length



from enum import Enum


class PredictionSortItemItemType0(str, Enum):
    ANOMALY_SCORE = "anomaly_score"
    CONFIDENCE = "confidence"
    TIMESTAMP = "timestamp"

    def __str__(self) -> str:
        return str(self.value)



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import Any, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="EmbedRequest")


@_attrs_define
class EmbedRequest:
    """
    Attributes:
        values (List[str]):
        max_seq_length (Union[None, Unset, int]):
    """

    values: List[str]
    max_seq_length: Union[None, Unset, int] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        values: List[str]
        if isinstance(self.values, list):
            values = self.values

        max_seq_length: Union[None, Unset, int]
        if isinstance(self.max_seq_length, Unset):
            max_seq_length = UNSET
        else:
            max_seq_length = self.max_seq_length

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "values": values,
            }
        )
        if max_seq_length is not UNSET:
            field_dict["max_seq_length"] = max_seq_length

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_values(data: object) -> List[str]:
            try:
                if not isinstance(data, list):
                    raise TypeError()
                values_type_0 = cast(List[str], data)

                return values_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                values_type_1 = cast(List[str], data)

                return values_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                values_type_2 = cast(List[str], data)

                return values_type_2
            except:  # noqa: E722
                pass
            if not isinstance(data, list):
                raise TypeError()
            values_type_3 = []
            _values_type_3 = data
            for values_type_3_item_data in _values_type_3:

                def _parse_values_type_3_item(data: object) -> str:
                    return cast(str, data)

                values_type_3_item = _parse_values_type_3_item(values_type_3_item_data)

                values_type_3.append(values_type_3_item)

            return values_type_3

        values = _parse_values(d.pop("values"))

        def _parse_max_seq_length(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        max_seq_length = _parse_max_seq_length(d.pop("max_seq_length", UNSET))

        embed_request = cls(
            values=values,
            max_seq_length=max_seq_length,
        )

        embed_request.additional_properties = d
        return embed_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import Any, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="LookupRequest")


@_attrs_define
class LookupRequest:
    """
    Attributes:
        query (List[str]):
        count (Union[Unset, int]):  Default: 1.
    """

    query: List[str]
    count: Union[Unset, int] = 1
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        query = self.query

        count = self.count

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "query": query,
            }
        )
        if count is not UNSET:
            field_dict["count"] = count

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        query = cast(List[str], d.pop("query"))

        count = d.pop("count", UNSET)

        lookup_request = cls(
            query=query,
            count=count,
        )

        lookup_request.additional_properties = d
        return lookup_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import Any, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="LabelClassMetrics")


@_attrs_define
class LabelClassMetrics:
    """
    Attributes:
        label (int):
        average_lookup_score (float):
        memory_count (int):
        label_name (Union[None, Unset, str]):
    """

    label: int
    average_lookup_score: float
    memory_count: int
    label_name: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        label = self.label

        average_lookup_score = self.average_lookup_score

        memory_count = self.memory_count

        label_name: Union[None, Unset, str]
        if isinstance(self.label_name, Unset):
            label_name = UNSET
        else:
            label_name = self.label_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "label": label,
                "average_lookup_score": average_lookup_score,
                "memory_count": memory_count,
            }
        )
        if label_name is not UNSET:
            field_dict["label_name"] = label_name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        label = d.pop("label")

        average_lookup_score = d.pop("average_lookup_score")

        memory_count = d.pop("memory_count")

        def _parse_label_name(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        label_name = _parse_label_name(d.pop("label_name", UNSET))

        label_class_metrics = cls(
            label=label,
            average_lookup_score=average_lookup_score,
            memory_count=memory_count,
            label_name=label_name,
        )

        label_class_metrics.additional_properties = d
        return label_class_metrics

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

import datetime
from typing import TYPE_CHECKING, Any, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.label_prediction_memory_lookup import LabelPredictionMemoryLookup
    from ..models.prediction_feedback import PredictionFeedback


T = TypeVar("T", bound="LabelPredictionWithMemoriesAndFeedback")


@_attrs_define
class LabelPredictionWithMemoriesAndFeedback:
    """
    Attributes:
        prediction_id (str):
        confidence (float):
        anomaly_score (Union[None, float]):
        label (int):
        label_name (Union[None, str]):
        timestamp (datetime.datetime):
        input_value (str):
        input_embedding (List[float]):
        logits (List[float]):
        expected_label (Union[None, int]):
        expected_label_name (Union[None, str]):
        memories (List['LabelPredictionMemoryLookup']):
        org_id (str):
        memoryset_id (str):
        model_id (str):
        updated_at (datetime.datetime):
        tags (List[str]):
        explanation (Union[None, str]):
        feedbacks (List['PredictionFeedback']):
    """

    prediction_id: str
    confidence: float
    anomaly_score: Union[None, float]
    label: int
    label_name: Union[None, str]
    timestamp: datetime.datetime
    input_value: str
    input_embedding: List[float]
    logits: List[float]
    expected_label: Union[None, int]
    expected_label_name: Union[None, str]
    memories: List["LabelPredictionMemoryLookup"]
    org_id: str
    memoryset_id: str
    model_id: str
    updated_at: datetime.datetime
    tags: List[str]
    explanation: Union[None, str]
    feedbacks: List["PredictionFeedback"]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        prediction_id = self.prediction_id

        confidence = self.confidence

        anomaly_score: Union[None, float]
        anomaly_score = self.anomaly_score

        label = self.label

        label_name: Union[None, str]
        label_name = self.label_name

        timestamp = self.timestamp.isoformat()

        input_value: str
        input_value = self.input_value

        input_embedding = self.input_embedding

        logits = self.logits

        expected_label: Union[None, int]
        expected_label = self.expected_label

        expected_label_name: Union[None, str]
        expected_label_name = self.expected_label_name

        memories = []
        for memories_item_data in self.memories:
            memories_item = memories_item_data.to_dict()
            memories.append(memories_item)

        org_id = self.org_id

        memoryset_id = self.memoryset_id

        model_id = self.model_id

        updated_at = self.updated_at.isoformat()

        tags = self.tags

        explanation: Union[None, str]
        explanation = self.explanation

        feedbacks = []
        for feedbacks_item_data in self.feedbacks:
            feedbacks_item = feedbacks_item_data.to_dict()
            feedbacks.append(feedbacks_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "prediction_id": prediction_id,
                "confidence": confidence,
                "anomaly_score": anomaly_score,
                "label": label,
                "label_name": label_name,
                "timestamp": timestamp,
                "input_value": input_value,
                "input_embedding": input_embedding,
                "logits": logits,
                "expected_label": expected_label,
                "expected_label_name": expected_label_name,
                "memories": memories,
                "org_id": org_id,
                "memoryset_id": memoryset_id,
                "model_id": model_id,
                "updated_at": updated_at,
                "tags": tags,
                "explanation": explanation,
                "feedbacks": feedbacks,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        from ..models.label_prediction_memory_lookup import LabelPredictionMemoryLookup
        from ..models.prediction_feedback import PredictionFeedback

        d = src_dict.copy()
        prediction_id = d.pop("prediction_id")

        confidence = d.pop("confidence")

        def _parse_anomaly_score(data: object) -> Union[None, float]:
            if data is None:
                return data
            return cast(Union[None, float], data)

        anomaly_score = _parse_anomaly_score(d.pop("anomaly_score"))

        label = d.pop("label")

        def _parse_label_name(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        label_name = _parse_label_name(d.pop("label_name"))

        timestamp = isoparse(d.pop("timestamp"))

        def _parse_input_value(data: object) -> str:
            return cast(str, data)

        input_value = _parse_input_value(d.pop("input_value"))

        input_embedding = cast(List[float], d.pop("input_embedding"))

        logits = cast(List[float], d.pop("logits"))

        def _parse_expected_label(data: object) -> Union[None, int]:
            if data is None:
                return data
            return cast(Union[None, int], data)

        expected_label = _parse_expected_label(d.pop("expected_label"))

        def _parse_expected_label_name(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        expected_label_name = _parse_expected_label_name(d.pop("expected_label_name"))

        memories = []
        _memories = d.pop("memories")
        for memories_item_data in _memories:
            memories_item = LabelPredictionMemoryLookup.from_dict(memories_item_data)

            memories.append(memories_item)

        org_id = d.pop("org_id")

        memoryset_id = d.pop("memoryset_id")

        model_id = d.pop("model_id")

        updated_at = isoparse(d.pop("updated_at"))

        tags = cast(List[str], d.pop("tags"))

        def _parse_explanation(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        explanation = _parse_explanation(d.pop("explanation"))

        feedbacks = []
        _feedbacks = d.pop("feedbacks")
        for feedbacks_item_data in _feedbacks:
            feedbacks_item = PredictionFeedback.from_dict(feedbacks_item_data)

            feedbacks.append(feedbacks_item)

        label_prediction_with_memories_and_feedback = cls(
            prediction_id=prediction_id,
            confidence=confidence,
            anomaly_score=anomaly_score,
            label=label,
            label_name=label_name,
            timestamp=timestamp,
            input_value=input_value,
            input_embedding=input_embedding,
            logits=logits,
            expected_label=expected_label,
            expected_label_name=expected_label_name,
            memories=memories,
            org_id=org_id,
            memoryset_id=memoryset_id,
            model_id=model_id,
            updated_at=updated_at,
            tags=tags,
            explanation=explanation,
            feedbacks=feedbacks,
        )

        label_prediction_with_memories_and_feedback.additional_properties = d
        return label_prediction_with_memories_and_feedback

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class PretrainedEmbeddingModelName(str, Enum):
    CDE_SMALL = "CDE_SMALL"
    CLIP_BASE = "CLIP_BASE"
    DISTILBERT = "DISTILBERT"
    GTE_BASE = "GTE_BASE"
    GTE_SMALL = "GTE_SMALL"

    def __str__(self) -> str:
        return str(self.value)



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

import datetime
from enum import Enum
from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.task_status import TaskStatus

if TYPE_CHECKING:
    from ..models.classification_evaluation_result import ClassificationEvaluationResult


T = TypeVar("T", bound="EvaluationResponse")


@_attrs_define
class EvaluationResponse:
    """
    Attributes:
        task_id (str):
        org_id (str):
        status (TaskStatus): Status of task in the task queue
        result (Union['ClassificationEvaluationResult', None]):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
    """

    task_id: str
    org_id: str
    status: TaskStatus
    result: Union["ClassificationEvaluationResult", None]
    created_at: datetime.datetime
    updated_at: datetime.datetime
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.classification_evaluation_result import ClassificationEvaluationResult

        task_id = self.task_id

        org_id = self.org_id

        status = self.status.value if isinstance(self.status, Enum) else self.status

        result: Union[Dict[str, Any], None]
        if isinstance(self.result, ClassificationEvaluationResult):
            result = self.result.to_dict()
        else:
            result = self.result

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "task_id": task_id,
                "org_id": org_id,
                "status": status,
                "result": result,
                "created_at": created_at,
                "updated_at": updated_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        from ..models.classification_evaluation_result import ClassificationEvaluationResult

        d = src_dict.copy()
        task_id = d.pop("task_id")

        org_id = d.pop("org_id")

        status = TaskStatus(d.pop("status"))

        def _parse_result(data: object) -> Union["ClassificationEvaluationResult", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                result_type_0 = ClassificationEvaluationResult.from_dict(data)

                return result_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ClassificationEvaluationResult", None], data)

        result = _parse_result(d.pop("result"))

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        evaluation_response = cls(
            task_id=task_id,
            org_id=org_id,
            status=status,
            result=result,
            created_at=created_at,
            updated_at=updated_at,
        )

        evaluation_response.additional_properties = d
        return evaluation_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

import datetime
from enum import Enum
from typing import Any, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.rac_head_type import RACHeadType

T = TypeVar("T", bound="RACModelMetadata")


@_attrs_define
class RACModelMetadata:
    """
    Attributes:
        id (str):
        org_id (str):
        name (str):
        description (Union[None, str]):
        version (int):
        num_classes (int):
        head_type (RACHeadType):
        memoryset_id (str):
        memory_lookup_count (int):
        weigh_memories (Union[None, bool]):
        min_memory_weight (Union[None, float]):
        storage_path (str):
        memoryset_collection_name (str):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
    """

    id: str
    org_id: str
    name: str
    description: Union[None, str]
    version: int
    num_classes: int
    head_type: RACHeadType
    memoryset_id: str
    memory_lookup_count: int
    weigh_memories: Union[None, bool]
    min_memory_weight: Union[None, float]
    storage_path: str
    memoryset_collection_name: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        org_id = self.org_id

        name = self.name

        description: Union[None, str]
        description = self.description

        version = self.version

        num_classes = self.num_classes

        head_type = self.head_type.value if isinstance(self.head_type, Enum) else self.head_type

        memoryset_id = self.memoryset_id

        memory_lookup_count = self.memory_lookup_count

        weigh_memories: Union[None, bool]
        weigh_memories = self.weigh_memories

        min_memory_weight: Union[None, float]
        min_memory_weight = self.min_memory_weight

        storage_path = self.storage_path

        memoryset_collection_name = self.memoryset_collection_name

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "org_id": org_id,
                "name": name,
                "description": description,
                "version": version,
                "num_classes": num_classes,
                "head_type": head_type,
                "memoryset_id": memoryset_id,
                "memory_lookup_count": memory_lookup_count,
                "weigh_memories": weigh_memories,
                "min_memory_weight": min_memory_weight,
                "storage_path": storage_path,
                "memoryset_collection_name": memoryset_collection_name,
                "created_at": created_at,
                "updated_at": updated_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        org_id = d.pop("org_id")

        name = d.pop("name")

        def _parse_description(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        description = _parse_description(d.pop("description"))

        version = d.pop("version")

        num_classes = d.pop("num_classes")

        head_type = RACHeadType(d.pop("head_type"))

        memoryset_id = d.pop("memoryset_id")

        memory_lookup_count = d.pop("memory_lookup_count")

        def _parse_weigh_memories(data: object) -> Union[None, bool]:
            if data is None:
                return data
            return cast(Union[None, bool], data)

        weigh_memories = _parse_weigh_memories(d.pop("weigh_memories"))

        def _parse_min_memory_weight(data: object) -> Union[None, float]:
            if data is None:
                return data
            return cast(Union[None, float], data)

        min_memory_weight = _parse_min_memory_weight(d.pop("min_memory_weight"))

        storage_path = d.pop("storage_path")

        memoryset_collection_name = d.pop("memoryset_collection_name")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        rac_model_metadata = cls(
            id=id,
            org_id=org_id,
            name=name,
            description=description,
            version=version,
            num_classes=num_classes,
            head_type=head_type,
            memoryset_id=memoryset_id,
            memory_lookup_count=memory_lookup_count,
            weigh_memories=weigh_memories,
            min_memory_weight=min_memory_weight,
            storage_path=storage_path,
            memoryset_collection_name=memoryset_collection_name,
            created_at=created_at,
            updated_at=updated_at,
        )

        rac_model_metadata.additional_properties = d
        return rac_model_metadata

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/endpoint_module.py.jinja

The main changes are:
- Update the API call responses to either return the successful response type or raise an error by:
    - Updating the _parse_response function to raise an error if the response status code is not in the 2xx range
- Inject a client into every method via a context manager
"""

from enum import Enum
from http import HTTPStatus
from typing import Any, Dict, List, Union

import httpx

from ...client import _client_context
from ...errors import get_error_for_response
from ...models.task import Task
from ...models.task_status import TaskStatus
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    status: Union[List[TaskStatus], None, TaskStatus, Unset] = UNSET,
    type: Union[List[str], None, Unset, str] = UNSET,
    limit: Union[Unset, int] = 100,
    offset: Union[Unset, int] = 0,
) -> dict[str, Any]:
    params: Dict[str, Any] = {}

    json_status: Union[List[str], None, Unset, str]
    if isinstance(status, Unset):
        json_status = UNSET
    elif isinstance(status, TaskStatus):
        json_status = status.value if isinstance(status, Enum) else status
    elif isinstance(status, list):
        json_status = []
        for status_type_1_item_data in status:
            status_type_1_item = (
                status_type_1_item_data.value if isinstance(status_type_1_item_data, Enum) else status_type_1_item_data
            )
            json_status.append(status_type_1_item)

    else:
        json_status = status
    params["status"] = json_status

    json_type: Union[List[str], None, Unset, str]
    if isinstance(type, Unset):
        json_type = UNSET
    elif isinstance(type, list):
        json_type = type

    else:
        json_type = type
    params["type"] = json_type

    params["limit"] = limit

    params["offset"] = offset

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/task",
        "params": params,
    }

    return _kwargs


def _parse_response(*, response: httpx.Response) -> List["Task"]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = Task.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if response.status_code == 401:
        raise get_error_for_response(response)
    if response.status_code == 500:
        raise get_error_for_response(response)
    if response.status_code == 503:
        raise get_error_for_response(response)
    if response.status_code == 404:
        raise get_error_for_response(response)
    else:
        raise get_error_for_response(response)


def _build_response(*, response: httpx.Response) -> Response[List["Task"]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    status: Union[List[TaskStatus], None, TaskStatus, Unset] = UNSET,
    type: Union[List[str], None, Unset, str] = UNSET,
    limit: Union[Unset, int] = 100,
    offset: Union[Unset, int] = 0,
) -> Response[List["Task"]]:
    """List Tasks

    Args:
        status (Union[List[TaskStatus], None, TaskStatus, Unset]):
        type (Union[List[str], None, Unset, str]):
        limit (Union[Unset, int]):  Default: 100.
        offset (Union[Unset, int]):  Default: 0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List['Task']]
    """

    kwargs = _get_kwargs(
        status=status,
        type=type,
        limit=limit,
        offset=offset,
    )

    with _client_context() as client:
        response = client.get_httpx_client().request(
            **kwargs,
        )

        return _build_response(response=response)


def sync(
    *,
    status: Union[List[TaskStatus], None, TaskStatus, Unset] = UNSET,
    type: Union[List[str], None, Unset, str] = UNSET,
    limit: Union[Unset, int] = 100,
    offset: Union[Unset, int] = 0,
) -> List["Task"]:
    """List Tasks

    Args:
        status (Union[List[TaskStatus], None, TaskStatus, Unset]):
        type (Union[List[str], None, Unset, str]):
        limit (Union[Unset, int]):  Default: 100.
        offset (Union[Unset, int]):  Default: 0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        List['Task']
    """

    return sync_detailed(
        status=status,
        type=type,
        limit=limit,
        offset=offset,
    ).parsed


async def asyncio_detailed(
    *,
    status: Union[List[TaskStatus], None, TaskStatus, Unset] = UNSET,
    type: Union[List[str], None, Unset, str] = UNSET,
    limit: Union[Unset, int] = 100,
    offset: Union[Unset, int] = 0,
) -> Response[List["Task"]]:
    """List Tasks

    Args:
        status (Union[List[TaskStatus], None, TaskStatus, Unset]):
        type (Union[List[str], None, Unset, str]):
        limit (Union[Unset, int]):  Default: 100.
        offset (Union[Unset, int]):  Default: 0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[List['Task']]
    """

    kwargs = _get_kwargs(
        status=status,
        type=type,
        limit=limit,
        offset=offset,
    )

    with _client_context() as client:
        response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    status: Union[List[TaskStatus], None, TaskStatus, Unset] = UNSET,
    type: Union[List[str], None, Unset, str] = UNSET,
    limit: Union[Unset, int] = 100,
    offset: Union[Unset, int] = 0,
) -> List["Task"]:
    """List Tasks

    Args:
        status (Union[List[TaskStatus], None, TaskStatus, Unset]):
        type (Union[List[str], None, Unset, str]):
        limit (Union[Unset, int]):  Default: 100.
        offset (Union[Unset, int]):  Default: 0.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        List['Task']
    """

    return (
        await asyncio_detailed(
            status=status,
            type=type,
            limit=limit,
            offset=offset,
        )
    ).parsed



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

import datetime
from enum import Enum
from typing import Any, List, Literal, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.filter_item_field_type_0_item import FilterItemFieldType0Item
from ..models.filter_item_field_type_2_item_type_1 import FilterItemFieldType2ItemType1
from ..models.filter_item_op import FilterItemOp

T = TypeVar("T", bound="FilterItem")


@_attrs_define
class FilterItem:
    """
    Attributes:
        field (Union[List[FilterItemFieldType0Item], List[Union[FilterItemFieldType2ItemType1, Literal['metrics']]],
            List[Union[Literal['metadata'], str]]]):
        op (FilterItemOp):
        value (Union[List[bool], List[float], List[int], List[str], None, bool, datetime.datetime, float, int, str]):
    """

    field: Union[
        List[FilterItemFieldType0Item],
        List[Union[FilterItemFieldType2ItemType1, Literal["metrics"]]],
        List[Union[Literal["metadata"], str]],
    ]
    op: FilterItemOp
    value: Union[List[bool], List[float], List[int], List[str], None, bool, datetime.datetime, float, int, str]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        field: Union[List[Union[Literal["metadata"], str]], List[Union[Literal["metrics"], str]], List[str]]
        if isinstance(self.field, list):
            field = []
            for field_type_0_item_data in self.field:
                field_type_0_item = (
                    field_type_0_item_data.value if isinstance(field_type_0_item_data, Enum) else field_type_0_item_data
                )
                field.append(field_type_0_item)

        op = self.op.value if isinstance(self.op, Enum) else self.op

        value: Union[List[bool], List[float], List[int], List[str], None, bool, float, int, str]
        if isinstance(self.value, datetime.datetime):
            value = self.value.isoformat()
        elif isinstance(self.value, list):
            value = self.value

        else:
            value = self.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "field": field,
                "op": op,
                "value": value,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_field(
            data: object,
        ) -> Union[
            List[FilterItemFieldType0Item],
            List[Union[FilterItemFieldType2ItemType1, Literal["metrics"]]],
            List[Union[Literal["metadata"], str]],
        ]:
            try:
                if not isinstance(data, list):
                    raise TypeError()
                field_type_0 = []
                _field_type_0 = data
                for field_type_0_item_data in _field_type_0:
                    field_type_0_item = FilterItemFieldType0Item(field_type_0_item_data)

                    field_type_0.append(field_type_0_item)

                return field_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                field_type_1 = []
                _field_type_1 = data
                for field_type_1_item_data in _field_type_1:

                    def _parse_field_type_1_item(data: object) -> Union[Literal["metadata"], str]:
                        field_type_1_item_type_0 = cast(Literal["metadata"], data)
                        if field_type_1_item_type_0 != "metadata":
                            raise ValueError(
                                f"field_type_1_item_type_0 must match const 'metadata', got '{field_type_1_item_type_0}'"
                            )
                        return field_type_1_item_type_0
                        return cast(Union[Literal["metadata"], str], data)

                    field_type_1_item = _parse_field_type_1_item(field_type_1_item_data)

                    field_type_1.append(field_type_1_item)

                return field_type_1
            except:  # noqa: E722
                pass
            if not isinstance(data, list):
                raise TypeError()
            field_type_2 = []
            _field_type_2 = data
            for field_type_2_item_data in _field_type_2:

                def _parse_field_type_2_item(data: object) -> Union[FilterItemFieldType2ItemType1, Literal["metrics"]]:
                    field_type_2_item_type_0 = cast(Literal["metrics"], data)
                    if field_type_2_item_type_0 != "metrics":
                        raise ValueError(
                            f"field_type_2_item_type_0 must match const 'metrics', got '{field_type_2_item_type_0}'"
                        )
                    return field_type_2_item_type_0
                    if not isinstance(data, str):
                        raise TypeError()
                    field_type_2_item_type_1 = FilterItemFieldType2ItemType1(data)

                    return field_type_2_item_type_1

                field_type_2_item = _parse_field_type_2_item(field_type_2_item_data)

                field_type_2.append(field_type_2_item)

            return field_type_2

        field = _parse_field(d.pop("field"))

        op = FilterItemOp(d.pop("op"))

        def _parse_value(
            data: object,
        ) -> Union[List[bool], List[float], List[int], List[str], None, bool, datetime.datetime, float, int, str]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                value_type_4 = isoparse(data)

                return value_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                value_type_5 = cast(List[str], data)

                return value_type_5
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                value_type_6 = cast(List[int], data)

                return value_type_6
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                value_type_7 = cast(List[float], data)

                return value_type_7
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                value_type_8 = cast(List[bool], data)

                return value_type_8
            except:  # noqa: E722
                pass
            return cast(
                Union[List[bool], List[float], List[int], List[str], None, bool, datetime.datetime, float, int, str],
                data,
            )

        value = _parse_value(d.pop("value"))

        filter_item = cls(
            field=field,
            op=op,
            value=value,
        )

        filter_item.additional_properties = d
        return filter_item

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/types.py.jinja

The main change is:
- Make Response's parsed type required
"""

from collections.abc import MutableMapping
from http import HTTPStatus
from typing import BinaryIO, Generic, Literal, Optional, TypeVar

from attrs import define

""" Contains some shared types for properties """


class Unset:
    def __bool__(self) -> Literal[False]:
        return False


UNSET: Unset = Unset()

FileJsonType = tuple[Optional[str], BinaryIO, Optional[str]]


@define
class File:
    """Contains information for file uploads"""

    payload: BinaryIO
    file_name: Optional[str] = None
    mime_type: Optional[str] = None

    def to_tuple(self) -> FileJsonType:
        """Return a tuple representation that httpx will accept for multipart/form-data"""
        return self.file_name, self.payload, self.mime_type


T = TypeVar("T")


@define
class Response(Generic[T]):
    """A response from an endpoint"""

    status_code: HTTPStatus
    content: bytes
    headers: MutableMapping[str, str]
    parsed: T


__all__ = ["UNSET", "File", "FileJsonType", "Response", "Unset"]



import random
from uuid import uuid4

import pytest
from datasets.arrow_dataset import Dataset

from .classification_model import ClassificationModel
from .datasource import Datasource
from .embedding_model import PretrainedEmbeddingModel
from .memoryset import LabeledMemoryset, TaskStatus


def test_create_memoryset(memoryset: LabeledMemoryset, hf_dataset: Dataset, label_names: list[str]):
    assert memoryset is not None
    assert memoryset.name == "test_memoryset"
    assert memoryset.embedding_model == PretrainedEmbeddingModel.GTE_BASE
    assert memoryset.label_names == label_names
    assert memoryset.insertion_status == TaskStatus.COMPLETED
    assert isinstance(memoryset.length, int)
    assert memoryset.length == len(hf_dataset)


def test_create_memoryset_unauthenticated(unauthenticated, datasource):
    with pytest.raises(ValueError, match="Invalid API key"):
        LabeledMemoryset.create("test_memoryset", datasource)


def test_create_memoryset_invalid_input(datasource):
    # invalid name
    with pytest.raises(ValueError, match=r"Invalid input:.*"):
        LabeledMemoryset.create("test memoryset", datasource)
    # invalid datasource
    datasource.id = str(uuid4())
    with pytest.raises(ValueError, match=r"Invalid input:.*"):
        LabeledMemoryset.create("test_memoryset_invalid_datasource", datasource)


def test_create_memoryset_already_exists_error(hf_dataset, label_names, memoryset):
    with pytest.raises(ValueError):
        LabeledMemoryset.from_hf_dataset("test_memoryset", hf_dataset, label_names=label_names, value_column="text")
    with pytest.raises(ValueError):
        LabeledMemoryset.from_hf_dataset(
            "test_memoryset", hf_dataset, label_names=label_names, value_column="text", if_exists="error"
        )


def test_create_memoryset_already_exists_open(hf_dataset, label_names, memoryset):
    # invalid label names
    with pytest.raises(ValueError):
        LabeledMemoryset.from_hf_dataset(
            memoryset.name,
            hf_dataset,
            label_names=["turtles", "frogs"],
            value_column="text",
            if_exists="open",
        )
    # different embedding model
    with pytest.raises(ValueError):
        LabeledMemoryset.from_hf_dataset(
            memoryset.name,
            hf_dataset,
            label_names=label_names,
            embedding_model=PretrainedEmbeddingModel.DISTILBERT,
            if_exists="open",
        )
    opened_memoryset = LabeledMemoryset.from_hf_dataset(
        memoryset.name,
        hf_dataset,
        embedding_model=PretrainedEmbeddingModel.GTE_BASE,
        if_exists="open",
    )
    assert opened_memoryset is not None
    assert opened_memoryset.name == memoryset.name
    assert opened_memoryset.length == len(hf_dataset)


def test_open_memoryset(memoryset, hf_dataset):
    fetched_memoryset = LabeledMemoryset.open(memoryset.name)
    assert fetched_memoryset is not None
    assert fetched_memoryset.name == memoryset.name
    assert fetched_memoryset.length == len(hf_dataset)


def test_open_memoryset_unauthenticated(unauthenticated, memoryset):
    with pytest.raises(ValueError, match="Invalid API key"):
        LabeledMemoryset.open(memoryset.name)


def test_open_memoryset_not_found():
    with pytest.raises(LookupError):
        LabeledMemoryset.open(str(uuid4()))


def test_open_memoryset_invalid_input():
    with pytest.raises(ValueError, match=r"Invalid input:.*"):
        LabeledMemoryset.open("not valid id")


def test_open_memoryset_unauthorized(unauthorized, memoryset):
    with pytest.raises(LookupError):
        LabeledMemoryset.open(memoryset.name)


def test_all_memorysets(memoryset):
    memorysets = LabeledMemoryset.all()
    assert len(memorysets) > 0
    assert any(memoryset.name == memoryset.name for memoryset in memorysets)


def test_all_memorysets_unauthenticated(unauthenticated):
    with pytest.raises(ValueError, match="Invalid API key"):
        LabeledMemoryset.all()


def test_all_memorysets_unauthorized(unauthorized, memoryset):
    assert memoryset not in LabeledMemoryset.all()


def test_drop_memoryset(hf_dataset):
    memoryset = LabeledMemoryset.from_hf_dataset(
        "test_memoryset_delete",
        hf_dataset.select(range(1)),
        value_column="text",
    )
    assert LabeledMemoryset.exists(memoryset.name)
    LabeledMemoryset.drop(memoryset.name)
    assert not LabeledMemoryset.exists(memoryset.name)


def test_drop_memoryset_unauthenticated(unauthenticated, memoryset):
    with pytest.raises(ValueError, match="Invalid API key"):
        LabeledMemoryset.drop(memoryset.name)


def test_drop_memoryset_not_found(memoryset):
    with pytest.raises(LookupError):
        LabeledMemoryset.drop(str(uuid4()))
    # ignores error if specified
    LabeledMemoryset.drop(str(uuid4()), if_not_exists="ignore")


def test_drop_memoryset_unauthorized(unauthorized, memoryset):
    with pytest.raises(LookupError):
        LabeledMemoryset.drop(memoryset.name)


def test_search(memoryset: LabeledMemoryset):
    memory_lookups = memoryset.search(["i love soup", "cats are cute"])
    assert len(memory_lookups) == 2
    assert len(memory_lookups[0]) == 1
    assert len(memory_lookups[1]) == 1
    assert memory_lookups[0][0].label == 0
    assert memory_lookups[1][0].label == 1


def test_search_count(memoryset: LabeledMemoryset):
    memory_lookups = memoryset.search("i love soup", count=3)
    assert len(memory_lookups) == 3
    assert memory_lookups[0].label == 0
    assert memory_lookups[1].label == 0
    assert memory_lookups[2].label == 0


def test_get_memory_at_index(memoryset: LabeledMemoryset, hf_dataset: Dataset, label_names: list[str]):
    memory = memoryset[0]
    assert memory.value == hf_dataset[0]["text"]
    assert memory.label == hf_dataset[0]["label"]
    assert memory.label_name == label_names[hf_dataset[0]["label"]]
    assert memory.source_id == hf_dataset[0]["source_id"]
    assert memory.score == hf_dataset[0]["score"]
    assert memory.key == hf_dataset[0]["key"]
    last_memory = memoryset[-1]
    assert last_memory.value == hf_dataset[-1]["text"]
    assert last_memory.label == hf_dataset[-1]["label"]


def test_get_range_of_memories(memoryset: LabeledMemoryset, hf_dataset: Dataset):
    memories = memoryset[1:3]
    assert len(memories) == 2
    assert memories[0].value == hf_dataset["text"][1]
    assert memories[1].value == hf_dataset["text"][2]


def test_get_memory_by_id(memoryset: LabeledMemoryset, hf_dataset: Dataset):
    memory = memoryset.get(memoryset[0].memory_id)
    assert memory.value == hf_dataset[0]["text"]
    assert memory == memoryset[memory.memory_id]


def test_get_memories_by_id(memoryset: LabeledMemoryset, hf_dataset: Dataset):
    memories = memoryset.get([memoryset[0].memory_id, memoryset[1].memory_id])
    assert len(memories) == 2
    assert memories[0].value == hf_dataset[0]["text"]
    assert memories[1].value == hf_dataset[1]["text"]


def test_query_memoryset(memoryset: LabeledMemoryset):
    memories = memoryset.query(filters=[("label", "==", 1)])
    assert len(memories) == 3
    assert all(memory.label == 1 for memory in memories)
    assert len(memoryset.query(limit=2)) == 2
    assert len(memoryset.query(filters=[("metadata.key", "==", "val1")])) == 1


def test_query_memoryset_with_feedback_metrics(model: ClassificationModel):
    prediction = model.predict("Do you love soup?")
    feedback_name = f"correct_{random.randint(0, 1000000)}"
    prediction.record_feedback(category=feedback_name, value=prediction.label == 0)
    memories = prediction.memoryset.query(filters=[("label", "==", 0)], with_feedback_metrics=True)
    assert len(memories) == 3
    assert all(memory.label == 0 for memory in memories)
    for memory in memories:
        assert memory.feedback_metrics is not None
        assert memory.feedback_metrics[feedback_name] is not None
        assert memory.feedback_metrics[feedback_name]["avg"] == 1.0
        assert memory.feedback_metrics[feedback_name]["count"] == 1
        assert isinstance(memory.lookup_count, int)


def test_query_memoryset_with_feedback_metrics_filter(model: ClassificationModel):
    prediction = model.predict("Do you love soup?")
    prediction.record_feedback(category="accurate", value=prediction.label == 0)
    memories = prediction.memoryset.query(
        filters=[("feedback_metrics.accurate.avg", ">", 0.5)], with_feedback_metrics=True
    )
    assert len(memories) == 3
    assert all(memory.label == 0 for memory in memories)
    for memory in memories:
        assert memory.feedback_metrics is not None
        assert memory.feedback_metrics["accurate"] is not None
        assert memory.feedback_metrics["accurate"]["avg"] == 1.0
        assert memory.feedback_metrics["accurate"]["count"] == 1


def test_query_memoryset_with_feedback_metrics_sort(model: ClassificationModel):
    prediction = model.predict("Do you love soup?")
    prediction.record_feedback(category="positive", value=1.0)
    prediction2 = model.predict("Do you like cats?")
    prediction2.record_feedback(category="positive", value=-1.0)
    memories = prediction.memoryset.query(
        filters=[("feedback_metrics.positive.avg", ">=", -1.0)],
        sort=[("feedback_metrics.positive.avg", "desc")],
        with_feedback_metrics=True,
    )
    assert len(memories) == prediction.memoryset.length
    assert memories[0].feedback_metrics["positive"]["avg"] == 1.0
    assert memories[-1].feedback_metrics["positive"]["avg"] == -1.0


def test_insert_memories(memoryset: LabeledMemoryset):
    memoryset.refresh()
    prev_length = memoryset.length
    memoryset.insert(
        [
            dict(value="tomato soup is my favorite", label=0),
            dict(value="cats are fun to play with", label=1),
        ]
    )
    assert memoryset.length == prev_length + 2
    memoryset.insert(dict(value="tomato soup is my favorite", label=0, key="test", source_id="test"))
    assert memoryset.length == prev_length + 3
    last_memory = memoryset[-1]
    assert last_memory.value == "tomato soup is my favorite"
    assert last_memory.label == 0
    assert last_memory.metadata
    assert last_memory.metadata["key"] == "test"
    assert last_memory.source_id == "test"


def test_update_memory(memoryset: LabeledMemoryset, hf_dataset: Dataset):
    memory_id = memoryset[0].memory_id
    updated_memory = memoryset.update(dict(memory_id=memory_id, value="i love soup so much"))
    assert updated_memory.value == "i love soup so much"
    assert updated_memory.label == hf_dataset[0]["label"]
    assert memoryset.get(memory_id).value == "i love soup so much"


def test_update_memory_instance(memoryset: LabeledMemoryset, hf_dataset: Dataset):
    memory = memoryset[0]
    updated_memory = memory.update(value="i love soup even more")
    assert updated_memory is memory
    assert memory.value == "i love soup even more"
    assert memory.label == hf_dataset[0]["label"]


def test_update_memories(memoryset: LabeledMemoryset):
    memory_ids = [memory.memory_id for memory in memoryset[:2]]
    updated_memories = memoryset.update(
        [
            dict(memory_id=memory_ids[0], value="i love soup so much"),
            dict(memory_id=memory_ids[1], value="cats are so cute"),
        ]
    )
    assert updated_memories[0].value == "i love soup so much"
    assert updated_memories[1].value == "cats are so cute"


def test_delete_memory(memoryset: LabeledMemoryset):
    memoryset.refresh()
    prev_length = memoryset.length
    memory_id = memoryset[0].memory_id
    memoryset.delete(memory_id)
    with pytest.raises(LookupError):
        memoryset.get(memory_id)
    assert memoryset.length == prev_length - 1


def test_delete_memories(memoryset: LabeledMemoryset):
    prev_length = memoryset.length
    memoryset.delete([memoryset[0].memory_id, memoryset[1].memory_id])
    assert memoryset.length == prev_length - 2


def test_clone_memoryset(memoryset: LabeledMemoryset):
    cloned_memoryset = memoryset.clone("test_cloned_memoryset", embedding_model=PretrainedEmbeddingModel.DISTILBERT)
    assert cloned_memoryset is not None
    assert cloned_memoryset.name == "test_cloned_memoryset"
    assert cloned_memoryset.length == memoryset.length
    assert cloned_memoryset.embedding_model == PretrainedEmbeddingModel.DISTILBERT
    assert cloned_memoryset.insertion_status == TaskStatus.COMPLETED


def test_embedding_evaluation(hf_dataset):
    datasource = Datasource.from_hf_dataset("eval_datasource", hf_dataset, if_exists="open")
    response = LabeledMemoryset.run_embedding_evaluation(
        datasource, embedding_models=["CDE_SMALL"], neighbor_count=2, value_column="text"
    )
    assert response is not None
    assert isinstance(response, dict)
    assert response is not None
    assert isinstance(response["evaluation_results"], list)
    assert len(response["evaluation_results"]) == 1
    assert response["evaluation_results"][0] is not None
    assert response["evaluation_results"][0]["embedding_model_name"] == "CDE_SMALL"
    assert response["evaluation_results"][0]["embedding_model_path"] == "OrcaDB/cde-small-v1"
    Datasource.drop("eval_datasource")



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/endpoint_module.py.jinja

The main changes are:
- Update the API call responses to either return the successful response type or raise an error by:
    - Updating the _parse_response function to raise an error if the response status code is not in the 2xx range
- Inject a client into every method via a context manager
"""

from http import HTTPStatus
from typing import Any, Dict, cast

import httpx

from ...client import _client_context
from ...errors import get_error_for_response
from ...models.list_predictions_request import ListPredictionsRequest
from ...types import Response


def _get_kwargs(
    *,
    body: ListPredictionsRequest,
) -> dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/telemetry/prediction/count",
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, response: httpx.Response) -> int:
    if response.status_code == 200:
        response_200 = cast(int, response.json())
        return response_200
    if response.status_code == 401:
        raise get_error_for_response(response)
    if response.status_code == 500:
        raise get_error_for_response(response)
    if response.status_code == 503:
        raise get_error_for_response(response)
    if response.status_code == 404:
        raise get_error_for_response(response)
    else:
        raise get_error_for_response(response)


def _build_response(*, response: httpx.Response) -> Response[int]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    body: ListPredictionsRequest,
) -> Response[int]:
    """Count Predictions

    Args:
        body (ListPredictionsRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[int]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    with _client_context() as client:
        response = client.get_httpx_client().request(
            **kwargs,
        )

        return _build_response(response=response)


def sync(
    *,
    body: ListPredictionsRequest,
) -> int:
    """Count Predictions

    Args:
        body (ListPredictionsRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        int
    """

    return sync_detailed(
        body=body,
    ).parsed


async def asyncio_detailed(
    *,
    body: ListPredictionsRequest,
) -> Response[int]:
    """Count Predictions

    Args:
        body (ListPredictionsRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[int]
    """

    kwargs = _get_kwargs(
        body=body,
    )

    with _client_context() as client:
        response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    body: ListPredictionsRequest,
) -> int:
    """Count Predictions

    Args:
        body (ListPredictionsRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        int
    """

    return (
        await asyncio_detailed(
            body=body,
        )
    ).parsed



"""This module contains internal utils for managing api keys in tests"""

import logging
import os
from typing import List

from dotenv import load_dotenv

from .._generated_api_client.api import (
    check_authentication,
    create_api_key,
    delete_api_key,
    delete_org,
    list_api_keys,
)
from .._generated_api_client.client import headers_context, set_base_url, set_headers
from .._generated_api_client.models import (
    CreateApiKeyRequest,
    CreateApiKeyRequestScopeItem,
)
from .._generated_api_client.models.api_key_metadata import ApiKeyMetadata
from .common import DropMode

load_dotenv()  # this needs to be here to ensure env is populated before accessing it
_ORCA_ROOT_ACCESS_API_KEY = os.environ.get("ORCA_ROOT_ACCESS_API_KEY", "00000000-0000-0000-0000-000000000000")


def _create_api_key(org_id: str, name: str, scopes: list[str] = ["ADMINISTER"]) -> str:
    """Creates an API key for the given organization"""
    with headers_context({"Api-Key": _ORCA_ROOT_ACCESS_API_KEY, "Org-Id": org_id}):
        res = create_api_key(
            body=CreateApiKeyRequest(name=name, scope=[CreateApiKeyRequestScopeItem(scope) for scope in scopes])
        )
        return res.api_key


def _list_api_keys(org_id: str) -> List[ApiKeyMetadata]:
    """Lists all API keys for the given organization"""
    with headers_context({"Api-Key": _ORCA_ROOT_ACCESS_API_KEY, "Org-Id": org_id}):
        return list_api_keys()


def _delete_api_key(org_id: str, name: str, if_not_exists: DropMode = "error") -> None:
    """Deletes the API key with the given name from the organization"""
    with headers_context({"Api-Key": _ORCA_ROOT_ACCESS_API_KEY, "Org-Id": org_id}):
        try:
            delete_api_key(name_or_id=name)
        except LookupError:
            if if_not_exists == "error":
                raise


def _delete_org(org_id: str) -> None:
    """Deletes the organization"""
    with headers_context({"Api-Key": _ORCA_ROOT_ACCESS_API_KEY, "Org-Id": org_id}):
        delete_org()


def _authenticate_local_api(org_id: str = "10e50000-0000-4000-a000-a78dca14af3a", api_key_name: str = "local") -> None:
    """Connect to the local API at http://localhost:1584/ and authenticate with a new API key"""
    set_base_url("http://localhost:1584/")
    _delete_api_key(org_id, api_key_name, if_not_exists="ignore")
    set_headers({"Api-Key": _create_api_key(org_id, api_key_name)})
    check_authentication()
    logging.info(f"Authenticated against local API at 'http://localhost:1584' with '{api_key_name}' API key")


__all__ = ["_create_api_key", "_delete_api_key", "_delete_org", "_list_api_keys", "_authenticate_local_api"]



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import Any, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.memoryset_cluster_analysis_config_clustering_method import MemorysetClusterAnalysisConfigClusteringMethod
from ..models.memoryset_cluster_analysis_config_partitioning_method import (
    MemorysetClusterAnalysisConfigPartitioningMethod,
)
from ..types import UNSET, Unset

T = TypeVar("T", bound="MemorysetClusterAnalysisConfig")


@_attrs_define
class MemorysetClusterAnalysisConfig:
    """
    Attributes:
        min_cluster_size (Union[None, Unset, int]):
        max_cluster_size (Union[None, Unset, int]):
        clustering_method (Union[Unset, MemorysetClusterAnalysisConfigClusteringMethod]):  Default:
            MemorysetClusterAnalysisConfigClusteringMethod.GRAPH.
        min_cluster_distance (Union[Unset, float]):  Default: 0.0.
        partitioning_method (Union[Unset, MemorysetClusterAnalysisConfigPartitioningMethod]):  Default:
            MemorysetClusterAnalysisConfigPartitioningMethod.NG.
        resolution (Union[None, Unset, float]):
        num_iterations (Union[Unset, int]):  Default: 2.
        random_state (Union[None, Unset, int]):  Default: 42.
    """

    min_cluster_size: Union[None, Unset, int] = UNSET
    max_cluster_size: Union[None, Unset, int] = UNSET
    clustering_method: Union[Unset, MemorysetClusterAnalysisConfigClusteringMethod] = (
        MemorysetClusterAnalysisConfigClusteringMethod.GRAPH
    )
    min_cluster_distance: Union[Unset, float] = 0.0
    partitioning_method: Union[Unset, MemorysetClusterAnalysisConfigPartitioningMethod] = (
        MemorysetClusterAnalysisConfigPartitioningMethod.NG
    )
    resolution: Union[None, Unset, float] = UNSET
    num_iterations: Union[Unset, int] = 2
    random_state: Union[None, Unset, int] = 42
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        min_cluster_size: Union[None, Unset, int]
        if isinstance(self.min_cluster_size, Unset):
            min_cluster_size = UNSET
        else:
            min_cluster_size = self.min_cluster_size

        max_cluster_size: Union[None, Unset, int]
        if isinstance(self.max_cluster_size, Unset):
            max_cluster_size = UNSET
        else:
            max_cluster_size = self.max_cluster_size

        clustering_method: Union[Unset, str] = UNSET
        if not isinstance(self.clustering_method, Unset):
            clustering_method = self.clustering_method.value

        min_cluster_distance = self.min_cluster_distance

        partitioning_method: Union[Unset, str] = UNSET
        if not isinstance(self.partitioning_method, Unset):
            partitioning_method = self.partitioning_method.value

        resolution: Union[None, Unset, float]
        if isinstance(self.resolution, Unset):
            resolution = UNSET
        else:
            resolution = self.resolution

        num_iterations = self.num_iterations

        random_state: Union[None, Unset, int]
        if isinstance(self.random_state, Unset):
            random_state = UNSET
        else:
            random_state = self.random_state

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if min_cluster_size is not UNSET:
            field_dict["min_cluster_size"] = min_cluster_size
        if max_cluster_size is not UNSET:
            field_dict["max_cluster_size"] = max_cluster_size
        if clustering_method is not UNSET:
            field_dict["clustering_method"] = clustering_method
        if min_cluster_distance is not UNSET:
            field_dict["min_cluster_distance"] = min_cluster_distance
        if partitioning_method is not UNSET:
            field_dict["partitioning_method"] = partitioning_method
        if resolution is not UNSET:
            field_dict["resolution"] = resolution
        if num_iterations is not UNSET:
            field_dict["num_iterations"] = num_iterations
        if random_state is not UNSET:
            field_dict["random_state"] = random_state

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()

        def _parse_min_cluster_size(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        min_cluster_size = _parse_min_cluster_size(d.pop("min_cluster_size", UNSET))

        def _parse_max_cluster_size(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        max_cluster_size = _parse_max_cluster_size(d.pop("max_cluster_size", UNSET))

        _clustering_method = d.pop("clustering_method", UNSET)
        clustering_method: Union[Unset, MemorysetClusterAnalysisConfigClusteringMethod]
        if isinstance(_clustering_method, Unset):
            clustering_method = UNSET
        else:
            clustering_method = MemorysetClusterAnalysisConfigClusteringMethod(_clustering_method)

        min_cluster_distance = d.pop("min_cluster_distance", UNSET)

        _partitioning_method = d.pop("partitioning_method", UNSET)
        partitioning_method: Union[Unset, MemorysetClusterAnalysisConfigPartitioningMethod]
        if isinstance(_partitioning_method, Unset):
            partitioning_method = UNSET
        else:
            partitioning_method = MemorysetClusterAnalysisConfigPartitioningMethod(_partitioning_method)

        def _parse_resolution(data: object) -> Union[None, Unset, float]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, float], data)

        resolution = _parse_resolution(d.pop("resolution", UNSET))

        num_iterations = d.pop("num_iterations", UNSET)

        def _parse_random_state(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        random_state = _parse_random_state(d.pop("random_state", UNSET))

        memoryset_cluster_analysis_config = cls(
            min_cluster_size=min_cluster_size,
            max_cluster_size=max_cluster_size,
            clustering_method=clustering_method,
            min_cluster_distance=min_cluster_distance,
            partitioning_method=partitioning_method,
            resolution=resolution,
            num_iterations=num_iterations,
            random_state=random_state,
        )

        memoryset_cluster_analysis_config.additional_properties = d
        return memoryset_cluster_analysis_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import TYPE_CHECKING, Any, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.memoryset_analysis_configs import MemorysetAnalysisConfigs


T = TypeVar("T", bound="MemorysetAnalysisRequest")


@_attrs_define
class MemorysetAnalysisRequest:
    """
    Attributes:
        configs (MemorysetAnalysisConfigs):
        lookup_count (Union[Unset, int]):  Default: 15.
        batch_size (Union[Unset, int]):  Default: 32.
        clear_metrics (Union[Unset, bool]):  Default: False.
    """

    configs: "MemorysetAnalysisConfigs"
    lookup_count: Union[Unset, int] = 15
    batch_size: Union[Unset, int] = 32
    clear_metrics: Union[Unset, bool] = False
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        configs = self.configs.to_dict()

        lookup_count = self.lookup_count

        batch_size = self.batch_size

        clear_metrics = self.clear_metrics

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "configs": configs,
            }
        )
        if lookup_count is not UNSET:
            field_dict["lookup_count"] = lookup_count
        if batch_size is not UNSET:
            field_dict["batch_size"] = batch_size
        if clear_metrics is not UNSET:
            field_dict["clear_metrics"] = clear_metrics

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        from ..models.memoryset_analysis_configs import MemorysetAnalysisConfigs

        d = src_dict.copy()
        configs = MemorysetAnalysisConfigs.from_dict(d.pop("configs"))

        lookup_count = d.pop("lookup_count", UNSET)

        batch_size = d.pop("batch_size", UNSET)

        clear_metrics = d.pop("clear_metrics", UNSET)

        memoryset_analysis_request = cls(
            configs=configs,
            lookup_count=lookup_count,
            batch_size=batch_size,
            clear_metrics=clear_metrics,
        )

        memoryset_analysis_request.additional_properties = d
        return memoryset_analysis_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
